#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.events import event_handler
from cocktail.pkgutils import import_object
from cocktail import schema
from cocktail.controllers import get_parameter, view_state, Location
from woost.models import (
    get_current_user,
    Site,
    ModifyPermission,
    CreatePermission
)
from woost.controllers.backoffice.useractions import get_user_action
from woost.controllers.backoffice.editstack import RelationNode
from woost.controllers.backoffice.editcontroller import EditController


class ItemFieldsController(EditController):

    section = "fields"
    form_prefix = "edited_item_"

    def __call__(self, *args, **kwargs):
        self._handle_form_data()
        return EditController.__call__(self, *args, **kwargs)

    def _handle_form_data(self):

        stack_node = self.stack_node
        form_data = stack_node.form_data
        translations = stack_node.translations

        section = self.params.read(
            schema.String("section", default = "fields")
        )

        added_translation = self.params.read(
            schema.String("add_translation",
                enumeration = self.available_languages
            )
        )

        deleted_translation = self.params.read(
            schema.String("delete_translation",
                enumeration = translations
            )
        )

        # Remove translations
        if deleted_translation:
            translations.remove(deleted_translation)
            for key, member in self.fields_schema.members().iteritems():
                if member.translated:
                    values = form_data.get(key)
                    if values:
                        values.pop(deleted_translation, None)

        get_method = cherrypy.request.method.upper() == "GET"

        # Load form data from the request
        get_parameter(
            self.fields_schema,
            target = form_data,
            languages = translations,
            prefix = self.form_prefix,
            errors = "ignore",
            implicit_booleans = not get_method,
            undefined = "skip" if get_method else "set_none"
        )

        # Add translations
        if added_translation and added_translation not in translations:
            translations.append(added_translation)
            translation_data = {}
            stack_node.content_type.translation.init_instance(translation_data)
            for key, value in translation_data.iteritems():
                schema.set(form_data, key, value, language = added_translation)

        # Drop references
        unlink = cherrypy.request.params.get("ItemSelector-unlink")

        if unlink:
            form_data[unlink] = None

        return form_data

    @cached_getter
    def fields_adapter(self):
        adapter = schema.Adapter()
        adapter.exclude([
            member.name
            for member in self.stack_node.content_type.members().itervalues()
            if (
                isinstance(member, schema.Collection)
                and not member.edit_inline
            )
            or (
                self.stack_node.item
                and self.stack_node.item.is_inserted
                and isinstance(member, (schema.RelationMember))
                and member.bidirectional and member.related_end.integral
            )
        ])
        return adapter

    @cached_getter
    def fields_schema(self):
        fields_schema = self.fields_adapter.export_schema(
            self.stack_node.form_schema
        )
        fields_schema.name = "BackOfficeEditForm"
        return fields_schema

    @cached_getter
    def output(self):
        output = EditController.output(self)
        output.update(
            submitted = self.submitted,
            available_languages = self.available_languages,
            fields_schema = self.fields_schema,
            selected_action = get_user_action("edit")
        )
        return output

    @cached_getter
    def view_class(self):
        return self.stack_node.item.edit_view

    @event_handler
    def handle_traversed(cls, event):

        # Restrict access to the edited item
        controller = event.source
        item = controller.stack_node.item
        user = get_current_user()

        if item.is_inserted:
            user.require_permission(ModifyPermission, target = item)
        else:
            user.require_permission(CreatePermission, target = item.__class__)

    @event_handler
    def handle_processed(cls, event):

        controller = event.source
        rel = cherrypy.request.params.get("ItemSelector-select")

        # Open the item selector
        if rel:
            pos = rel.find("-")
            root_content_type_name = rel[:pos]
            selection_parameter = rel[pos + 1:]
            key = selection_parameter[len(controller.form_prefix):]

            # Push the relation as a new stack node
            current_node = controller.stack_node
            rel_node = RelationNode()
            rel_node.member = current_node.content_type[key]
            controller.edit_stack.push(rel_node)

            value = schema.get(current_node.form_data, key)

            raise cherrypy.HTTPRedirect(
                controller.context["cms"].contextual_uri("content")
                + "?" + view_state(
                    selection = value.id if value is not None else None,
                    edit_stack = controller.edit_stack.to_param(),
                    client_side_scripting = controller.client_side_scripting
                )
            )

        # Open an editor for a new nested item
        new = cherrypy.request.params.get("ItemSelector-new")

        if new:
            pos = new.find("-")
            member_name = new[:pos]
            content_type_name = new[pos + 1:]

            # Push the relation as a new stack node
            current_node = controller.stack_node
            rel_node = RelationNode()
            rel_node.member = current_node.content_type[member_name]
            controller.edit_stack.push(rel_node)

            raise cherrypy.HTTPRedirect(
                controller.context["cms"].contextual_uri(
                    "content",
                    "new",
                    item_type = content_type_name,
                    edit_stack = controller.edit_stack.to_param()
                )
            )

        # Open an editor for an existing nested item
        edit = cherrypy.request.params.get("ItemSelector-edit")

        if edit:
            raise cherrypy.HTTPRedirect(
                controller.edit_uri(controller.stack_node.form_data[edit])
            )




#-*- coding: utf-8 -*-
u"""Defines migrations to the database schema for woost.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import when
from cocktail.persistence import MigrationStep

def admin_members_restriction(members):

    def add_permission(e):

        from woost.models import Role, ModifyMemberPermission

        everybody_role = Role.require_instance(qname = "woost.everybody")
        permission = ModifyMemberPermission(
            matching_members = list(members),
            authorized = False
        )
        permission.insert()

        for i, p in enumerate(everybody_role.permissions):
            if isinstance(p, ModifyMemberPermission) and p.authorized:
                everybody_role.permissions.insert(i, permission)
                break
        else:
            everybody_role.permissions.append(permission)

    return add_permission

#------------------------------------------------------------------------------

step = MigrationStep("added woost.models.Document.robots_*")

step.executing.append(
    admin_members_restriction([
        "woost.models.document.Document.robots_should_index",
        "woost.models.document.Document.robots_should_follow"
    ])
)

@step.processor("woost.models.document.Document")
def set_defaults(document):
    if not hasattr(document, "_robots_should_index"):
        document.robots_should_index = True
        document.robots_should_follow = True

#------------------------------------------------------------------------------

step = MigrationStep("added woost.models.Publishable.requires_https")

step.executing.append(
    admin_members_restriction([
        "woost.models.publishable.Publishable.requires_https"
    ])
)

@step.processor("woost.models.publishable.Publishable")
def set_defaults(publishable):
    if not hasattr(publishable, "_requires_https"):
        publishable.requires_https = False


#------------------------------------------------------------------------------

step = MigrationStep("make Product extend Publishable")

@when(step.executing)
def update_keys(e):
    from woost.extensions.shop import ShopExtension

    if ShopExtension.enabled:
        from cocktail.translations import translations
        from woost.models import Publishable, Controller, Language
        from woost.extensions.shop import create_product_controller
        from woost.extensions.shop.product import Product

        # Update the publishable keys
        Publishable.keys.update([product.id for product in Product.select()])

        # Create the product controller
        create_product_controller()

#------------------------------------------------------------------------------

step = MigrationStep("use TranslationMapping to translations")

@when(step.executing)
def update_translations(e):
    from cocktail.schema import TranslationMapping
    from cocktail.persistence import PersistentObject

    def translated_items(schema):
        if schema.translated and schema.indexed:
            for item in schema.select():
                yield item
        else:
            for derived_schema in schema.derived_schemas(False):
                for item in translated_items(derived_schema):
                    yield item

    for item in translated_items(PersistentObject):
        translations = TranslationMapping(
            owner = item,
            items = item.translations._items
        )
        item.translations._items = translations
        item._p_changed = True

    PersistentObject.rebuild_indexes(True)

#------------------------------------------------------------------------------

step = MigrationStep(
    "rename EmailTemplate.embeded_images to EmailTemplate.attachments"
)

step.rename_member(
    "woost.models.EmailTemplate",
    "embeded_images",
    "attachments"
)

#------------------------------------------------------------------------------

step = MigrationStep(
    "Apply full text indexing to elements with no translations"
)

@when(step.executing)
def rebuild_full_text_index(e):
    from woost.models import Item
    Item.rebuild_full_text_indexes(True)

#------------------------------------------------------------------------------

step = MigrationStep(
    "Replace EmailTemplate.attachments with EmailTemplate.initialization_code"
)

@when(step.executing)
def relocate_attachments_code(e):
    from woost.models import EmailTemplate

    for email_template in EmailTemplate.select():
        code = getattr(email_template, "_attachments", None)
        if code:
            del email_template._attachments
            if email_template.initialization_code:
                code = email_template.initialization_code + "\n\n" + code
            email_template.initialization_code = code

#------------------------------------------------------------------------------

step = MigrationStep(
    "Added the Role.implicit member"
)

@when(step.executing)
def flag_implicit_roles(e):
    from woost.models import Role

    implicit_roles_qnames = set((
        "woost.anonymous",
        "woost.everybody",
        "woost.authenticated"
    ))

    for role in Role.select():
        role.implicit = (role.qname in implicit_roles_qnames)




#-*- coding: utf-8 -*-
u"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from simplejson import dumps
from cocktail.translations import get_language
from cocktail.html import Element, templates
from woost.models import Template
from woost.extensions.recaptcha import ReCaptchaExtension


class ReCaptchaBox(Element):

    API_SSL_SERVER = "https://www.google.com/recaptcha/api"
    API_SERVER = "http://www.google.com/recaptcha/api"

    tag = None

    def __init__(self, *args, **kwargs):
        Element.__init__(self, *args, **kwargs)
        self.recaptcha_params = {
            "public_key": ReCaptchaExtension.instance.public_key,
            "api_server": self.API_SSL_SERVER if ReCaptchaExtension.instance.ssl
                          else self.API_SERVER,
            "error_param": "&amp;error=%s" % ("",)
        }
        self.recaptcha_options = {}

    def _ready(self):

        Element._ready(self)

        options = self.recaptcha_options.copy()

        language = get_language()

        options["lang"] = language
        if language == "ca":
            options["lang"] = "es"
            options["custom_translations"] = {
                "instructions_visual": "Escriu les 2 paraules:",
                "instructions_audio": "Escriu el que sentis:",
                "play_again": "Tornar a escoltar",
                "cant_hear_this": "Descarregar so en MP3",
                "visual_challenge": "Obtindre un repte visual",
                "audio_challenge": "Obtindre un repte audible",
                "refresh_btn": "Obtindre un nou repte",
                "help_btn": "Ajuda",
                "incorrect_try_again": "Incorrecte. Torna a provar-ho."
            }
        elif language not in ("en", "nl", "fr", "de", "pt", "ru", "es", "tr"):
            options["lang"] = "en"

        if ReCaptchaExtension.instance.theme == "custom":
            options.setdefault(
                    "custom_theme_widget",
                    ReCaptchaExtension.instance.custom_theme_widget
            )

        options.setdefault("theme", ReCaptchaExtension.instance.theme)

        init_options = Element("script")
        init_options["type"] = "text/javascript"
        init_options.append("var RecaptchaOptions = %s;" % dumps(options))

        init_script = Element()
        init_script.append("""<script type="text/javascript" src="%(api_server)s/challenge?k=%(public_key)s%(error_param)s"></script>

        <noscript>
          <iframe src="%(api_server)s/noscript?k=%(public_key)s%(error_param)s" style="height: 300px; width: 500px; border: 0;" ></iframe><br />
          <textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
          <input type='hidden' name='recaptcha_response_field' value='manual_challenge' />
        </noscript>
        """ % self.recaptcha_params)

        self.append(init_options)

        if ReCaptchaExtension.instance.theme == "custom":
            custom_container = templates.new(
                ReCaptchaExtension.instance.custom_template
            )
            custom_container.widgetid = ReCaptchaExtension.instance.custom_theme_widget
            self.append(custom_container.render())

        self.append(init_script)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# UserForm
#------------------------------------------------------------------------------
translations.define("UserForm",
    ca = u"Formulari",
    es = u"Formulario",
    en = u"Form"
)

translations.define("UserForm-plural",
    ca = u"Formularis",
    es = u"Formularios",
    en = u"Forms"
)

translations.define("UserForm.form",
    ca = u"Formulari",
    es = u"Formulario",
    en = u"Form"
)

translations.define("UserForm.form_model",
    ca = u"Model",
    es = u"Modelo",
    en = u"Model"
)

translations.define("UserForm.form_model-explanation",
    ca = u"El model de dades utilitzat per generar el formulari",
    es = u"El modelo de datos utilizado para generar el formulario",
    en = u"The data model used to generate the form"
)

translations.define("UserForm.excluded_members",
    ca = u"Membres exclosos",
    es = u"Miembros excluídos",
    en = u"Excluded members"
)

translations.define("UserForm.excluded_members-explanation",
    ca = u"Membres del model que no han d'aparèixer al formulari",
    es = u"Miembros del modelo que no deben aparecer en el formulario",
    en = u"Model members that shouldn't appear on the form"
)

translations.define("UserForm.should_save_instances",
    ca = u"Desar registres",
    es = u"Guardar registros",
    en = u"Save instances"
)

translations.define("UserForm.should_save_instances-explanation",
    ca = u"Indica si les dades introduïdes pels usuaris al formulari es desen "
         u"a la base de dades",
    es = u"Indica si los datos introducidos por los usuarios en el formulario "
         u"se guardan en la base de datos",
    en = u"Indicates if submitting the form generates a new register in the "
         u"data base"
)

translations.define("UserForm.redirection",
    ca = u"Pàgina de confirmació",
    es = u"Página de confirmación",
    en = u"Confirmation page"
)

translations.define("UserForm.redirection-explanation",
    ca = u"Un document al que es redirigirà els usuaris després que completin "
         u"el formulari satisfactòriament",
    es = u"Un documento al que se redirigirá a los usuarios tras rellenar el "
         u"formulario satisfactoriamente",
    en = u"A document where users will be redirected to after they have "
         u"sucessfully submitted the form"
)

translations.define("UserForm.email_notifications",
    ca = u"Notificacions per email",
    es = u"Notificaciones por correo",
    en = u"Email notifications"
)

translations.define("woost.extensions.usermodels.user_form_controller.title",
    ca = u"Controlador de formulari",
    es = u"Controlador de formulario",
    en = u"Form controller"
)

translations.define("woost.extensions.usermodels.user_form_template.title",
    ca = u"Plantilla de formulari",
    es = u"Plantilla de formulario",
    en = u"Form template"
)

# UserMember
#------------------------------------------------------------------------------
translations.define("UserMember",
    ca = u"Membre",
    es = u"Miembro",
    en = u"Member"
)

translations.define("UserMember-plural",
    ca = u"Membres",
    es = u"Miembros",
    en = u"Members"
)

translations.define("UserMember.description",
    ca = u"Descripció",
    es = u"Descripción",
    en = u"Description"
)

translations.define("UserMember.definition",
    ca = u"Definició",
    es = u"Definición",
    en = u"Definition"
)

translations.define("UserMember.indexing",
    ca = u"Indexat",
    es = u"Indexado",
    en = u"Indexing"
)

translations.define("UserMember.constraints",
    ca = u"Restriccions",
    es = u"Restricciones",
    en = u"Constraints"
)

translations.define("UserMember.behavior",
    ca = u"Comportament",
    es = u"Comportamiento",
    en = u"Behavior"
)

translations.define("UserMember.code",
    ca = u"Codi personalitzat",
    es = u"Código personalizado",
    en = u"Custom code"
)

translations.define("UserMember.label",
    ca = u"Nom descriptiu",
    es = u"Nombre descriptivo",
    en = u"Descriptive name"
)

translations.define("UserMember.explanation",
    ca = u"Explicació",
    es = u"Explicación",
    en = u"Explanation"
)

translations.define("UserMember.explanation-explanation",
    ca = u"Aclaracions sobre el membre que es mostraran als formularis",
    es = u"Aclaracionens sobre el miembro que aparecerán en los formularios",
    en = u"Additional notes on the member's nature and purpose, will be "
         u"displayed on forms"
)

translations.define("UserMember.member_descriptive",
    ca = u"És descriptiu",
    es = u"Es descriptivo",
    en = u"Is descriptive"
)

translations.define("UserMember.member_descriptive-explanation",
    ca = u"El membre descriptiu d'un model és el que s'utilitza quan cal "
         u"descriure una instància del model als usuaris (als llistats, "
         u"formularis, etc)",
    es = u"El miembro descriptivo de un modelo es el que se utiliza cuando es "
         u"necesario describir una instancia del modelo a los usuarios (en "
         u"listados, formularios, etc)",
    en = u"A model's descriptive member is used to describe instances of the "
         u"model to users (in listings, forms, etc)"
)

translations.define("UserMember.member_name",
    ca = u"Nom intern",
    es = u"Nombre interno",
    en = u"Internal name"
)

translations.define("UserMember.member_name-explanation",
    ca = u"Identificador únic del membre, d'ús intern per l'aplicació",
    es = u"Identificador único del miembro, de uso interno para la aplicación",
    en = u"Unique identifier for the member, used internally by the "
         u"application"
)

translations.define("UserMember.member_translated",
    ca = u"Traduït",
    es = u"Traducido",
    en = u"Translated"
)

translations.define("UserMember.member_versioned",
    ca = u"Versionat",
    es = u"Versionado",
    en = u"Versioned"
)

translations.define("UserMember.member_versioned-explanation",
    ca = u"Indica si es manté un registre del valor del membre al llarg del "
         u"temps",
    es = u"Indica si se mantiene un registro del valor del miembro a lo largo "
         u"del tiempo",
    en = u"Indicates if the application should keep a history of the values "
         u"assigned to the member"
)

translations.define("UserMember.member_indexed",
    ca = u"Indexat",
    es = u"Indexado",
    en = u"Indexed"
)

translations.define("UserMember.member_indexed-explanation",
    ca = u"Accelera les cerques basades en aquest membre, a costa de "
         u"penalitzar-ne la modificació. Establir només si el membre serà "
         u"cercat freqüentment.",
    es = u"Acelera las búsquedas basadas en este miembro, a costa de "
         u"penalizar su modificación. Establecer solo si el miembro será "
         u"usado frecuentemente como criterio de búsqueda.",
    en = u"Greatly accelerates searches based on this member, but makes "
         u"updates more costly. Use only on members which will be frequently "
         u"used as search criteria."
)

translations.define("UserMember.initialization",
    ca = u"Codi d'inicialització",
    es = u"Código de inicialización",
    en = u"Initialization code"
)

translations.define("UserMember.member_unique",
    ca = u"Exigeix valor únic",
    es = u"Exige valor único",
    en = u"Requires unique value"
)

translations.define("UserMember.member_member_group",
    ca = u"Grup",
    es = u"Grupo",
    en = u"Group"
)

translations.define("UserMember.member_required",
    ca = u"Obligatori",
    es = u"Obligatorio",
    en = u"Required"
)

translations.define("UserMember.member_default",
    ca = u"Valor per defecte",
    es = u"Valor por defecto",
    en = u"Default value"
)

translations.define("UserMember.member_enumeration",
    ca = u"Valors vàlids",
    es = u"Valores válidos",
    en = u"Valid values"
)

translations.define("UserMember.member_listed_by_default",
    ca = u"Apareix al llistat per defecte",
    es = u"Aparece en el listado por defecto",
    en = u"Included in the default listing"
)

translations.define("UserMember.member_editable",
    ca = u"Editable",
    es = u"Editable",
    en = u"Editable"
)

translations.define("UserMember.member_edit_control",
    ca = u"Control d'edició",
    es = u"Control de edición",
    en = u"Edit control"
)

translations.define("UserMember.member_searchable",
    ca = u"Cercable",
    es = u"Admite búsquedas",
    en = u"Searchable"
)

translations.define("UserMember.member_search_control",
    ca = u"Control per les cerques",
    es = u"Control para las búsquedas",
    en = u"Search control"
)

for model_name in (
    "UserInteger",
    "UserFloat",
    "UserDecimal",
    "UserFraction",
    "UserDate",
    "UserTime",
    "UserDateTime"
):
    translations.define(model_name + ".member_min",
        ca = u"Valor mínim",
        es = u"Valor mínimo",
        en = u"Minimum value"
    )
    translations.define(model_name + ".member_max",
        ca = u"Valor màxim",
        es = u"Valor máximo",
        en = u"Maximum value"
    )

# UserBoolean
#------------------------------------------------------------------------------
translations.define("UserBoolean",
    ca = u"Cert o fals",
    es = u"Cierto o falso",
    en = u"True or false"
)

translations.define("UserBoolean-plural",
    ca = u"Cert o fals",
    es = u"Cierto o falso",
    en = u"True or false"
)

# UserInteger
#------------------------------------------------------------------------------
translations.define("UserInteger",
    ca = u"Número enter",
    es = u"Número entero",
    en = u"Integer number"
)

translations.define("UserInteger-plural",
    ca = u"Números enters",
    es = u"Números enteros",
    en = u"Integer numbers"
)

# UserFloat
#------------------------------------------------------------------------------
translations.define("UserFloat",
    ca = u"Número de coma flotant",
    es = u"Número de coma flotante",
    en = u"Floating point number"
)

translations.define("UserFloat-plural",
    ca = u"Números de coma flotant",
    es = u"Números de coma flotante",
    en = u"Floating point numbers"
)

# UserDecimal
#------------------------------------------------------------------------------
translations.define("UserDecimal",
    ca = u"Número decimal",
    es = u"Número decimal",
    en = u"Decimal number"
)

translations.define("UserDecimal-plural",
    ca = u"Números decimals",
    es = u"Números decimales",
    en = u"Decimal numbers"
)

# UserFraction
#------------------------------------------------------------------------------
translations.define("UserFraction",
    ca = u"Número fraccional",
    es = u"Número fraccional",
    en = u"Fractional number"
)

translations.define("UserFraction-plural",
    ca = u"Números fraccionals",
    es = u"Números fraccionales",
    en = u"Fractional numbers"
)

# UserString
#------------------------------------------------------------------------------
translations.define("UserString",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("UserString-plural",
    ca = u"Textos",
    es = u"Textos",
    en = u"Texts"
)

translations.define("UserString.member_min",
    ca = u"Longitud mínima",
    es = u"Longitud mínima",
    en = u"Minimum length"
)

translations.define("UserString.member_max",
    ca = u"Longitud màxima",
    es = u"Longitud máxima",
    en = u"Maximum length"
)

translations.define("UserString.member_format",
    ca = u"Format",
    es = u"Formato",
    en = u"Format"
)

translations.define("UserString.member_text_search",
    ca = u"Incloure a les cerques de text",
    es = u"Incluir en las búsquedas de texto",
    en = u"Include in full text search"
)

translations.define("UserString.member_normalized_index",
    ca = u"Index normalitzat",
    es = u"Índice normalizado",
    en = u"Normalized index"
)

# UserDate
#------------------------------------------------------------------------------
translations.define("UserDate",
    ca = u"Data",
    es = u"Fecha",
    en = u"Date"
)

translations.define("UserDate-plural",
    ca = u"Dates",
    es = u"Fechas",
    en = u"Dates"
)

# UserDate
#------------------------------------------------------------------------------
translations.define("UserTime",
    ca = u"Hora",
    es = u"Hora",
    en = u"Time"
)

translations.define("UserTime-plural",
    ca = u"Hores",
    es = u"Horas",
    en = u"Times"
)

# UserDateTime
#------------------------------------------------------------------------------
translations.define("UserDateTime",
    ca = u"Data i hora",
    es = u"Fecha y hora",
    en = u"Date and time"
)

translations.define("UserDateTime-plural",
    ca = u"Data i hora",
    es = u"Fecha y hora",
    en = u"Date and time"
)

# UserRelation
#------------------------------------------------------------------------------
translations.define("UserRelation",
    ca = u"Relació",
    es = u"Relación",
    en = u"Relation"
)

translations.define("UserRelation-plural",
    ca = u"Relacions",
    es = u"Relaciones",
    en = u"Relations"
)

translations.define("UserRelation.member_related_key",
    ca = u"Nom del membre relacionat",
    es = u"Nombre del miembro relacionado",
    en = u"Related key"
)

translations.define("UserRelation.member_bidirectional",
    ca = u"Bidireccional",
    es = u"Bidirectional",
    en = u"Bidireccional"
)

translations.define("UserRelation.member_integral",
    ca = u"Integral",
    es = u"Integral",
    en = u"Integral"
)

translations.define("UserRelation.member_cascade_delete",
    ca = u"Eliminació en cascada",
    es = u"Eliminación en cascada",
    en = u"Cascade delete"
)

translations.define("UserRelation.member_text_search",
    ca = u"Incloure a les cerques de text",
    es = u"Incluir en las búsquedas de texto",
    en = u"Include in full text search"
)

# UserReference
#------------------------------------------------------------------------------
translations.define("UserReference",
    ca = u"Referència",
    es = u"Referencia",
    en = u"Reference"
)

translations.define("UserReference-plural",
    ca = u"Referències",
    es = u"Referencias",
    en = u"References"
)

translations.define("UserReference.member_type",
    ca = u"Tipus d'objecte",
    es = u"Tipo de objeto",
    en = u"Object type"
)

translations.define("UserReference.member_class_family",
    ca = u"Referència de classe",
    es = u"Referencia de clase",
    en = u"Class reference"
)

translations.define("UserReference.member_cycles_allowed",
    ca = u"Permetre cicles",
    es = u"Permitir ciclos",
    en = u"Cycles allowed",
)

# UserCollection
#------------------------------------------------------------------------------
translations.define("UserCollection",
    ca = u"Col·lecció",
    es = u"Colección",
    en = u"Collection"
)

translations.define("UserCollection-plural",
    ca = u"Col·leccions",
    es = u"Collections",
    en = u"Collection"
)

translations.define("UserCollection.member_items",
    ca = u"Elements de la col·lecció",
    es = u"Elementos de la colección",
    en = u"Collection items"
)

translations.define("UserCollection.member_min",
    ca = u"Mida mínima",
    es = u"Tamaño mínimo",
    en = u"Minimum size"
)

translations.define("UserCollection.member_max",
    ca = u"Mida màxima",
    es = u"Tamaño máximo",
    en = u"Maximum size"
)

translations.define("UserCollection.member_items",
    ca = u"Elements de la col·lecció",
    es = u"Elementos de la colección",
    en = u"Collection items"
)

# UserModel
#------------------------------------------------------------------------------
translations.define("UserModel",
    ca = u"Model d'usuari",
    es = u"Modelo de usuario",
    en = u"User model"
)

translations.define("UserModel-plural",
    ca = u"Models d'usuari",
    es = u"Modelos de usuario",
    en = u"User models"
)

translations.define("UserModel.plural_label",
    ca = u"Nom descriptiu en plural",
    es = u"Nombre descriptivo en plural",
    en = u"Plural descriptive name"
)

translations.define("UserModel.package_name",
    ca = u"Nom del paquet",
    es = u"Nombre del paquete",
    en = u"Package name"
)

translations.define("UserModel.base_model",
    ca = u"Model base",
    es = u"Modelo base",
    en = u"Base model"
)

translations.define("UserModel.child_members",
    ca = u"Membres",
    es = u"Miembros",
    en = u"Members"
)

translations.define("UserModel.member_instantiable",
    ca = u"Instanciable",
    es = u"Instanciable",
    en = u"Instantiable"
)

translations.define("UserModel.member_visible_from_root",
    ca = u"Visible al llistat principal",
    es = u"Visible en el listado principal",
    en = u"Visible in the main listing"
)

# UI controls
#------------------------------------------------------------------------------
translations.define("woost.extensions.usermodels.auto-control",
    ca = u"Automàtic",
    es = u"Automático",
    en = u"Automatic"
)

translations.define("cocktail.html.TextBox",
    ca = u"Caixa de text",
    es = u"Caja de texto",
    en = u"Text box"
)

translations.define("cocktail.html.PasswordBox",
    ca = u"Caixa de contrasenya",
    es = u"Caja de contraseña",
    en = u"Password box"
)

translations.define("cocktail.html.TextArea",
    ca = u"Caixa de text multilínia",
    es = u"Caja de texto multilinea",
    en = u"Multiline text box"
)

translations.define("woost.views.RichTextEditor",
    ca = u"Editor de text avançat",
    es = u"Editor de texto avanzado",
    en = u"Rich text editor"
)

translations.define("cocktail.html.CheckBox",
    ca = u"Casella de verificació",
    es = u"Casilla de verificación",
    en = u"Check box"
)

translations.define("cocktail.html.RadioSelector",
    ca = u"Botons de radio",
    es = u"Botones de radio",
    en = u"Radio buttons"
)

translations.define("cocktail.html.CheckList",
    ca = u"Llistat de caselles de verificació",
    es = u"Listado de casillas de verificación",
    en = u"Check list"
)

translations.define("cocktail.html.MultipleChoiceSelector",
    ca = u"Quadre de diàleg amb múltiples opcions",
    es = u"Ventana modal con múltiples opciones",
    en = u"Multiple choices dialog window"
)

translations.define("cocktail.html.DropdownSelector",
    ca = u"Selector desplegable",
    es = u"Selector desplegable",
    en = u"Dropdown selector"
)

translations.define("woost.views.ItemSelector",
    ca = u"Selector d'element relacionat",
    es = u"Selector de elemento relacionado",
    en = u"Related item selector"
)

translations.define("cocktail.html.DatePicker",
    ca = u"Selector de data/hora",
    es = u"Selector de fecha/hora",
    en = u"Date/time selector"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
import os
from tempfile import mkdtemp
from zipfile import ZipFile
from shutil import copyfileobj
from cocktail.modeling import cached_getter
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import get_parameter
from cocktail.controllers.fileupload import FileUpload
from woost.models import (
    get_current_user,
    changeset_context,
    File
)
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class UploadFilesController(BaseBackOfficeController):

    view_class = "woost.views.BackOfficeUploadFilesView"
    imported_files = None

    @cached_getter
    def form_schema(self):

        form_schema = schema.Schema("UploadFilesForm")

        upload = FileUpload("upload", required = True)
        upload["mime_type"].enumeration = [
            "application/zip",
            "application/x-zip-compressed"
        ]
        form_schema.add_member(upload)

        return form_schema

    @cached_getter
    def form_data(self):
        return get_parameter(self.form_schema, errors = "ignore")

    @cached_getter
    def form_errors(self):
        return schema.ErrorList(
            self.form_schema.get_errors(self.form_data)
                if self.submitted
                else []
        )

    @cached_getter
    def valid(self):
        return not self.form_errors

    @cached_getter
    def submitted(self):
        return cherrypy.request.method == "POST"

    def submit(self):

        self.imported_files = []

        file = self.form_data["upload"]["file"]
        zip_file = ZipFile(file)

        upload_path = self.context["cms"].upload_path
        temp_dir = mkdtemp()

        try:
            with changeset_context(get_current_user()):
                for file_path in zip_file.namelist():

                    file_name = os.path.basename(file_path)

                    # Skip directories
                    if not file_name:
                        continue

                    source = zip_file.open(file_path)
                    temp_file = os.path.join(temp_dir, file_name)
                    dest = open(temp_file, "wb")
                    copyfileobj(source, dest)
                    source.close()
                    dest.close()

                    try:
                        imported_file = File.from_path(
                            temp_file,
                            upload_path
                        )
                        imported_file.insert()
                        self.imported_files.append(imported_file)
                    finally:
                        os.remove(temp_file)

            datastore.commit()
        finally:
            os.rmdir(temp_dir)

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            form_data = self.form_data,
            form_schema = self.form_schema,
            form_errors = self.form_errors,
            imported_files = self.imported_files
        )
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from urllib import urlopen, quote_plus
from simplejson import loads
from cocktail.persistence import datastore
from cocktail.translations import translations
from cocktail.controllers import Location
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.facebookpublication.facebookpublicationtarget \
    import FacebookPublicationTarget


class FBPublishAuthUserAction(UserAction):

    content_type = FacebookPublicationTarget

    included = frozenset(["item_buttons"])
    excluded = frozenset(["new_item"])
    min = 1
    max = 1

    def invoke(self, controller, selection):

        publication_target = selection[0]
        code = cherrypy.request.params.get("code")
        error = cherrypy.request.params.get("error")
        error_reason = cherrypy.request.params.get("error_reason")
        error_description = cherrypy.request.params.get("error_description")

        # Authorization failed
        if error:
            notify_user(
                translations(
                    "woost.extensions.facebookauthentication."
                    "fbpublish_auth.error",
                ),
                category = "error",
                transient = False
            )

        # Authorization successful
        elif code:
            location = Location.get_current(relative = False)
            del location.query_string["code"]

            auth_url = (
                "https://graph.facebook.com/oauth/access_token?"
                "client_id=%s"
                "&redirect_uri=%s"
                "&client_secret=%s"
                "&code=%s"
                % (
                    publication_target.app_id,
                    quote_plus(str(location)),
                    publication_target.app_secret,
                    code
                )
            )
            response = urlopen(auth_url)
            response_status = response.getcode()
            response_body = response.read()

            if response_status < 200 or response_status > 299:
                notify_user(
                    translations(
                        "woost.extensions.facebookauthentication."
                        "fbpublish_auth.oauth_error",
                        response = response_body
                    ),
                    category = "error",
                    transient = False
                )
                return

            auth_token = response_body.split("&")[0].split("=")[1]

            # Facebook pages require an additional authentication step
            fb_page_id = cherrypy.request.params.get("fb_page")

            if fb_page_id:
                accounts_url = (
                    "https://graph.facebook.com/%s/accounts?access_token=%s"
                    % (publication_target.administrator_id, auth_token)
                )
                response_data = urlopen(accounts_url).read()
                json = loads(response_data)

                for account in json["data"]:
                    if account["id"] == fb_page_id:
                        auth_token = account["access_token"]
                        break
                else:
                    auth_token = None

            publication_target.auth_token = auth_token
            datastore.commit()

            notify_user(
                translations(
                    "woost.extensions.facebookauthentication."
                    "fbpublish_auth.success"
                ),
                category = "success"
            )

        # Begin authorization request
        else:
            # Determine if the selected graph URL is a Facebook Page
            graph_url = "http://graph.facebook.com/%s?metadata=1" \
                      % publication_target.graph_object_id
            response_data = urlopen(graph_url).read()
            json = loads(response_data)
            fb_page_id = json["type"] == "page" and json["id"]

            location = Location.get_current(relative = False)
            location.query_string["item_action"] = self.id

            permissions = "publish_stream,offline_access,user_photos"

            if fb_page_id:
                location.query_string["fb_page"] = fb_page_id
                permissions += ",manage_pages"

            auth_url = (
                "https://www.facebook.com/dialog/oauth?"
                "client_id=%s"
                "&redirect_uri=%s"
                "&scope=%s"
                % (
                    publication_target.app_id,
                    quote_plus(str(location)),
                    permissions
                )
            )
            raise cherrypy.HTTPRedirect(auth_url)

FBPublishAuthUserAction("fbpublish_auth").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from cocktail.modeling import getter
from woost.models import Document
from woost.extensions.blocks.block import Block


class MenuBlock(Block):

    instantiable = True
    view_class = "woost.views.Menu"

    root = schema.Reference(
        type = Document,
        related_end = schema.Collection(),
        member_group = "content"
    )

    root_visible = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    max_depth = schema.Integer(
        member_group = "content"
    )

    expanded = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.root = self.root
        view.root_visible = self.root_visible
        view.max_depth = self.max_depth
        view.expanded = self.expanded




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.events import when
from cocktail.translations import (
    translations,
    set_language
)
from cocktail import schema
from cocktail.persistence import datastore
from woost.models import Extension, Language, Controller

translations.define("ShopExtension",
    ca = u"Botiga",
    es = u"Tienda",
    en = u"Shop"
)

translations.define("ShopExtension-plural",
    ca = u"Botigues",
    es = u"Tiendas",
    en = u"Shops"
)


class ShopExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona els elements necessaris per implementar una botiga
            electrònica.""",
            "ca"
        )
        self.set("description",
            u"""Proporciona los elementos necesarios para implementar una
            tienda electrónica.""",
            "es"
        )
        self.set("description",
            u"""Supplies the building blocks required to implement an online
            shop.""",
            "en"
        )

    def _load(self):

        from woost.extensions import shop
        from woost.extensions.shop import (
            strings,
            product,
            productcategory,
            shoporder,
            shoporderentry,
            pricing,
            basket,
            userfilter
        )

        for module, keys in (
            (product, ("Product",)),
            (productcategory, ("ProductCategory",)),
            (shoporder, ("ShopOrder",)),
            (shoporderentry, ("ShopOrderEntry",)),
            (pricing, (
                "PricingPolicy",
                "Discount",
                "PriceOverride",
                "RelativeDiscount",
                "PercentageDiscount",
                "FreeUnitsDiscount",
                "ShippingCost",
                "ShippingCostOverride",
                "CumulativeShippingCost",
                "Tax",
                "CumulativeTax",
                "PercentageTax"
            )),
            (basket, ("Basket",))
        ):
            for key in keys:
                setattr(shop, key, getattr(module, key))

        ShopExtension.add_member(
            schema.Collection("discounts",
                items = schema.Reference(type = pricing.Discount),
                related_end = schema.Reference()
            )
        )

        ShopExtension.add_member(
            schema.Collection("shipping_costs",
                items = schema.Reference(type = pricing.ShippingCost),
                related_end = schema.Reference()
            )
        )

        ShopExtension.add_member(
            schema.Collection("taxes",
                items = schema.Reference(type = pricing.Tax),
                related_end = schema.Reference()
            )
        )

        from tpv import (
            Currency,
            Payment,
            PaymentItem,
            PaymentNotFoundError
        )
        from woost.extensions.payments import PaymentsExtension
        from woost.extensions.payments.paymentgateway import PaymentGateway
        from woost.extensions.payments.transactionnotifiedtrigger \
            import launch_transaction_notification_triggers

        payments_ext = PaymentsExtension.instance

        def get_payment(self, payment_id):

            order = shoporder.ShopOrder.get_instance(int(payment_id))

            if order is None:
                raise PaymentNotFoundError(payment_id)

            payment = Payment()
            payment.id = order.id
            payment.amount = order.cost
            payment.shop_order = order
            payment.currency = Currency(payments_ext.payment_gateway.currency)

            for entry in order.entries:
                payment.add(PaymentItem(
                    reference = str(entry.product.id),
                    description = translations(entry.product),
                    units = entry.quantity,
                    price = entry.cost
                ))

            return payment

        PaymentGateway.get_payment = get_payment

        def receive_order_payment(event):
            payment = event.payment
            shop_order = payment.shop_order
            set_language(shop_order.language)
            shop_order.status = payment.status
            shop_order.gateway_parameters = payment.gateway_parameters

        def commit_order_payment(event):
            datastore.commit()

        events = PaymentGateway.transaction_notified
        pos = events.index(launch_transaction_notification_triggers)
        events.insert(pos, receive_order_payment)
        events.insert(pos + 2, commit_order_payment)

        self.install()

    def _install(self):

        # Create the product controller
        controller = Controller()
        controller.qname = "woost.product_controller"
        for language in Language.codes:
            value = translations(
                "woost.extensions.shop Product controller title",
                language
            )
            if value:
                controller.set("title", value, language)
        controller.python_name = \
            "woost.extensions.shop.productcontroller.ProductController"
        controller.insert()




#-*- coding: utf-8 -*-
u"""

@author:        Jordi Fernández
@contact:       jordi.fernandez@whads.com
@organization:  Whads/Accent SL
@since:         March 2010
"""
from __future__ import with_statement
import os
import ftplib
from shutil import copy
from datetime import datetime
from tempfile import mkstemp
from zipfile import ZipFile
from cocktail import schema
from cocktail.events import EventHub, Event
from cocktail.modeling import abstractmethod
from cocktail.translations import get_language
from cocktail.persistence import PersistentMapping
from cocktail.controllers import context as controller_context
from woost import app
from woost.models import Item, File, get_current_user
from woost.models.file import file_hash
from woost.models.changesets import changeset_context


class StaticSiteDestination(Item):
    """A class tasked with publishing a static snapshot of a site's content to
    a concrete location.

    This is mostly an abstract class, meant to be extended by subclasses. Each
    subclass should implement exporting to a concrete kind of location or
    media. Subclasses must implement L{write_file} and L{create_folder}, and
    probably L{setup}.

    @var chunk_size: The number of bytes written at once by L{write_file}.
    @type chunk_size: int
    """
    instantiable = False
    visible_from_root = False
    chunk_size = 1024 * 8

    destination_permissions = schema.Collection(
        items = "woost.extensions.staticsite.exportationpermission."
                "ExportationPermission",
        related_key = "destination",
        bidirectional = True
    )

    encoding = schema.String(
        required = True,
        default = "utf-8",
        text_search = False
    )


    def __init__(self, *args, **kwargs):
        Item.__init__(self, *args, **kwargs)
        self.__last_export_hashes = PersistentMapping()

    def setup(self, context):
        """Prepares the exporter for an export process.

        The object of this method is to allow exporters to perform any
        initialization they require before writing files to their destination.

        @param context: A dictionary where the exporter can place any
            contextual information it many need throgout the export process. It
            will be made available to all L{write_file} calls.
        @type context: dict
        """

    def cleanup(self, context):
        """Frees resources after an export operation.

        This method is guaranteed to be called after the export operation is
        over, even if an exception was raised during its execution.

        @param context: The dictionary used by the exporter during the export
            process to maintain its contextual information.
        @type context: dict
        """

    def _export(
        self,
        context,
        status_tracker = None
    ):
        if status_tracker:
            status_tracker.beginning(
                context = context
            )

        for file, file_path in context["snapshoter"].snapshot(
            context["selection"],
            context = context
        ):
            exported = None

            try:
                exported = self.export_file(
                    file,
                    file_path,
                    context
                )
            except Exception, error:

                handled = False

                if status_tracker:
                    e = status_tracker.file_processed(
                        file = file_path,
                        status = "failed",
                        context = context,
                        error = error,
                        error_handled = False
                    )
                    handled = e.error_handled

                if not handled:
                    raise

            else:
                if status_tracker:
                    status_tracker.file_processed(
                        file = file_path,
                        status = "exported" if exported else "not_modified",
                        context = context,
                        error = None,
                        error_handled = False
                    )

    def export(
        self,
        snapshoter,
        selection,
        update_only = False,
        status_tracker = None,
        context = {}
    ):
        """Exports site snapshot to this destination.

        @param snapshoter: The snapshoter used by the exporter during the export
            process.
        @type snapshoter: StaticSiteSnapShoter

        @param update_only: When set to True, items will only be exported if
            they have pending changes that have not been exported to this
            destination yet.
        @type update_only: bool

        @param status_tracker: An object to report events to during the export
            process.
        @type status_tracker: L{StatusTracker}

        @param context: A dictionary used to share any contextual information
            with the exporter.
        @type context: dict

        @return: The dictionary used by the exporter during the export
            process to maintain its contextual information.
        @rtype: dict
        """

        controller_context["exporting_static_site"] = True
        context["existing_folders"] = set()
        context.update(
            snapshoter = snapshoter,
            selection = selection,
            update_only = update_only
        )
        self.setup(context)

        try:
            self._export(
                context,
                status_tracker = status_tracker
            )
        finally:
            del controller_context["exporting_static_site"]
            self.cleanup(context)

        return context

    def export_file(self,
        file,
        file_path,
        context):
        """Exports a file.

        @param file: The file's contents file to export. Can be specified using
            a file-like object, or a filesystem path.
        @type file: file-like or str

        @param file_path: The file path, relative to the destination's root.
        @type file_path: str

        @param context: A dictionary where the exporter can place any
            contextual information it many need throgout the export process. It
            will be made available to all L{write_file} calls.
        @type context: dict

        @return: True if the item is exported, False if L{update_only} is set
            to True and the item has no changes to export.
        @rtype: bool
        """

        update_only = context.get("update_only")
        hash = file_hash(file)

        if update_only \
        and hash == self.__last_export_hashes.get(file_path):
            return False

        folder, filename = os.path.split(file_path)

        self.create_path(folder, context)
        self.write_file(file, file_path, context)
        self.__last_export_hashes[file_path] = hash
        return True


    def create_path(self, relative_path, context):
        """Recursively creates all folders in the given path.

        The method will invoke L{create_folder} for each folder in the provided
        path.

        @param relative_path: The path to create, relative to the destination's
            root.
        @type relative_path: unicode

        @param context: The dictionary used by the exporter to maintain any
            contextual information it many need throgout the export process.

            If a context is provided and it contains an 'existing_folders' key,
            it should be a set of strings representing paths that are known to
            be present at the destination. If given, folders in the set will be
            skipped, and those that are created will be added to the set. This
            mechanism can be useful when calling this method repeatedly, to
            avoid unnecessary calls to L{create_folder}.
        @type existing_folders: unicode set
        """
        existing_folders = context.get("existing_folders")

        def ascend(folder):
            if folder and \
            (existing_folders is None or folder not in existing_folders):
                ascend(os.path.dirname(folder))
                created = self.create_folder(folder, context)
                if existing_folders and created:
                    existing_folders.add(folder)

        ascend(relative_path)

    @abstractmethod
    def create_folder(self, relative_path, context):
        """Creates a folder on the destination, if it doesn't exist yet.

        @param relative_path: The path of the folder to create, relative to the
            destination's root.
        @type relative_path: unicode

        @param context: The dictionary used by the exporter to maintain any
            contextual information it many need throgout the export process.
        @type context: dict

        @return: True if the folder didn't exist, and was created, False if it
            already existed.
        @rtype: bool
        """

    @abstractmethod
    def write_file(self, file, path, context):
        """Writes a file to the destination.

        @param file: The file to write. Can be a given as a path to a local
            file, or as a file-like object.
        @type file: str or file-like object

        @param path: The relative path within the exporter's designated
            destination where the file should be written. Must include the
            file's name (and extension, if any).
        @type path: str

        @param context: A dictionary used by the exporter to maintain
            contextual information across its operations (ie. an open
            connection to an FTP server, an instance of a ZipFile class, etc).
        @type context: dict
        """

    def view_class(self):
        return "woost.extensions.staticsite.ExportStaticSiteView"

    def output(self):
        return {}


class StatusTracker(object):
    """An object that allows clients to be kept up to date of the progress of
    an L{export operation<StaticSiteDestination.export>}.
    """
    __metaclass__ = EventHub

    beginning = Event(
        """An event triggered after an exporter has finished its preparations
        and is ready to start the export operation.

        @ivar context: The L{context<StaticSiteDestination.setup.context>}
            used by the export operation to maintain its state.
        @type context: dict
        """)

    file_processed = Event(
        """An event triggered after file has been processed.

        This event will be triggered at least once for each generated file on
        the exportation process.

        The event will be triggered regardless of the outcome of the exporting
        process, and even if the item is discarded and not exported. Clients
        can differentiate between these cases using the L{status} attribute.

        @ivar file: The generated file path.
        @type file: str

        @ivar status: The status for the processed file. Will be one of the
            following:
                - ignored: The file is not published or marked as not able
                  to be exported, and has been skipped.
                - not-modified: The file hasn't been modified since the last
                  time it was exported, and the operation has been flagged as
                  "L{update only<StaticSiteDestination.export.update_only>}".
                - exported: The file has been correctly exported.
                - failed: The export attempt raised an exception.
        @type status: str

        @ivar error: Only relevant if L{status} is 'failed'. A reference
            to the exception raised while trying to export the item.
        @type error: L{Exception}

        @ivar error_handled: Only relevant if L{status} is 'failed'. Allows
            the event response code to capture an export error. If set to True,
            the exporter will continue
        @type error_handled: bool

        @ivar context: The L{context<StaticSiteDestination.setup.context>}
            used by the export operation to maintain its state.
        @type context: dict
        """)


class FolderDestination(StaticSiteDestination):
    """A class that exports a static snapshot of a site's content to a local
    folder.
    """
    instantiable = True

    target_folder = schema.String(
        required = True,
        unique = True,
        descriptive = True,
        text_search = False
    )

    def create_folder(self, folder, context):
        full_path = os.path.join(self.target_folder, folder)
        if not os.path.exists(full_path):
            os.mkdir(full_path)
            return True

        return False

    def write_file(self, file, path, context):

        full_path = os.path.join(self.target_folder, path)

        # Copy local files
        if isinstance(file, basestring):
            copy(file, full_path)

        # Save data from file-like objects
        else:
            target_file = open(full_path, "wb")
            try:
                while True:
                    chunk = file.read(self.chunk_size)
                    if not chunk:
                        break
                    target_file.write(chunk)
            finally:
                target_file.close()


class FTPDestination(StaticSiteDestination):
    """A class that exports a static snapshot of a site's content to an FTP
    server.
    """
    instantiable = True

    members_order = [
        "ftp_host",
        "ftp_user",
        "ftp_password",
        "ftp_path"
    ]

    ftp_host = schema.String(
        required = True,
        text_search = False
    )

    ftp_user = schema.String(
        text_search = False
    )

    ftp_password = schema.String(
        edit_control = "cocktail.html.PasswordBox",
        text_search = False
    )

    ftp_path = schema.String(
        text_search = False
    )

    def __translate__(self, language, **kwargs):

        if self.draft_source is None:

            desc = self.ftp_host
            if desc:

                user = self.ftp_user
                if user:
                    desc = user + "@" + desc

                path = self.ftp_path
                if path:
                    if not path[0] == "/":
                        path = "/" + path
                    desc += path

                return "ftp://" + desc

        return StaticSiteDestination.__translate__(self, language, **kwargs)

    def setup(self, context):
        context["ftp"] = ftplib.FTP(
            self.ftp_host,
            self.ftp_user,
            self.ftp_password
        )

    def cleanup(self, context):
        context["ftp"].quit()

    def write_file(self, file, path, context):

        ftp = context["ftp"]
        path = self._get_ftp_path(path)

        # Remove existing files
        try:
            ftp.delete(path)
        except ftplib.error_perm:
            pass

        # Handle both local files and file-like objects
        if isinstance(file, basestring):
            file = open(file, "r")
            should_close = True
        else:
            should_close = False

        try:
            ftp.storbinary("STOR " + path, file, self.chunk_size)
        finally:
            if should_close:
                file.close()

    def create_folder(self, folder, context):

        ftp = context["ftp"]
        path = self._get_ftp_path(folder)

        if not self._path_exists(ftp, path):
            ftp.mkd(path)
            return True

        return False

    def _get_ftp_path(self, *args):

        path = self.ftp_path

        if not path:
            path = "/"
        elif path[-1] != "/":
            path += "/"

        path = path + "/".join(arg.strip("/") for arg in args)

        try:
            return path.encode(self.encoding)
        except UnicodeEncodeError:
            return path

    def _path_exists(self, ftp, path):

        path = path.rstrip("/")

        if not path:
            return True

        pos = path.rfind("/")

        if pos == -1:
            parent = "."
            name = path
        else:
            parent = path[:pos]
            name = path[pos + 1:]

        if not parent.endswith("/"):
            parent = parent + "/"

        return (parent + name) in ftp.nlst(parent)


class ZipDestination(StaticSiteDestination):
    """A class that exports a static snapshot of a site's content to a ZIP
    file.
    """
    instantiable = True

    def setup(self, context):
        self.root_folder = app.path("static", "export")
        if not os.path.exists(self.root_folder):
            os.mkdir(self.root_folder)

        context["file_name"] = \
            "%s.zip" % datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
        zip_path = os.path.join(
            self.root_folder,
            context["file_name"]
        )
        context["zip_file"] = ZipFile(zip_path, 'w')

    def _export(
        self,
        context,
        status_tracker = None
    ):
        StaticSiteDestination._export(
            self,
            context,
            status_tracker = status_tracker
        )

        # Close the zip file
        context["zip_file"].close()

    def create_folder(self, folder, context):
        return True

    def write_file(self, file, path, context):

        if isinstance(file, basestring):
            context["zip_file"].write(file, path)
        else:
            try:
                context["zip_file"].writestr(path, file.read())
            finally:
                file.close()

    def view_class(self, context):
        return "woost.extensions.staticsite.ExportStaticSiteZipView"

    def output(self, context):
        output = StaticSiteDestination.output(self)
        output.update(
            file_name = context["file_name"]
        )
        return output




#-*- coding: utf-8 -*-
u"""
Declaration of back office actions.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
import cherrypy
from cocktail.modeling import getter, ListWrapper
from cocktail.translations import translations
from cocktail.controllers import view_state
from cocktail import schema
from woost.models import (
    Item,
    Publishable,
    URI,
    File,
    get_current_user,
    CreatePermission,
    ModifyPermission,
    DeletePermission,
    ConfirmDraftPermission,
    ReadHistoryPermission
)
from woost.controllers.backoffice.editstack import (
    SelectionNode,
    RelationNode
)

# User action model declaration
#------------------------------------------------------------------------------

# Class stub (needed by the metaclass)
UserAction = None

_action_list = ListWrapper([])
_action_map = {}

def get_user_action(id):
    """Gets a user action, given its unique identifier.

    @param id: The unique identifier of the action to obtain.
    @type id: str

    @return: The requested user action, or None if not defined.
    @rtype: L{UserAction}
    """
    return _action_map.get(id)

def get_user_actions(**kwargs):
    """Returns a collection of all actions registered with the site.

    @return: The list of user actions.
    @rtype: iterable L{UserAction} sequence
    """
    return _action_list

def get_view_actions(context, target = None):
    """Obtains the list of actions that can be displayed on a given view.

    @param context: A set of string identifiers, such as "context_menu",
        "toolbar", etc. Different views can make use of as many different
        identifiers as they require.
    @type container: str set

    @param target: The item or content type affected by the action.
    @type target: L{Item<woost.models.item.Item>} instance or class

    @return: The list of user actions available under the specified context.
    @rtype: iterable L{UserAction} sequence
    """
    return (
        action
        for action in _action_list
        if action.enabled
            and action.is_available(context, target)
    )

def add_view_action_context(view, clue):
    """Adds contextual information to the given view, to be gathered by
    L{get_view_actions_context} and passed to L{get_view_actions}.

    @param view: The view that gains the context clue.
    @type view: L{Element<cocktail.html.element.Element>}

    @param clue: The context identifier added to the view.
    @type clue: str
    """
    view_context = getattr(view, "actions_context", None)
    if view_context is None:
        view_context = set()
        view.actions_context = view_context
    view_context.add(clue)

def get_view_actions_context(view):
    """Extracts clues on the context that applies to a given view and its
    ancestors, to supply to the L{get_view_actions} function.

    @param view: The view to inspect.
    @type view: L{Element<cocktail.html.element.Element>}

    @return: The set of context identifiers for the view and its ancestors.
    @rtype: str set
    """
    context = set()

    while view:
        view_context = getattr(view, "actions_context", None)
        if view_context:
            context.update(view_context)
        view = view.parent

    return context


class UserAction(object):
    """An action that is made available to users at the backoffice
    interface. The user actions model allows site implementors to extend the
    site with their own actions, or to disable or override standard actions
    with fine grained control of their context.

    @ivar enabled: Controls the site-wide availavility of the action.
    @type enabled: bool

    @ivar included: A set of context identifiers under which the action is
        made available. Entries on the sequence are joined using a logical OR.
        Entries can also consist of a tuple of identifiers, which will be
        joined using a logical AND operation.
    @type included: set(str or tuple(str))

    @ivar excluded: A set of context identifiers under which the action won't
        be made available. Identifiers are specified using the same format used
        by the L{included} parameter. If both X{included} and X{excluded} are
        specified, both conditions will be tested, with X{excluded} carrying
        more weight.
    @type excluded: set(str or tuple(str))

    @ivar content_type: When set, the action will only be made available to the
        indicated content type or its subclasses.
    @type content_type: L{Item<woost.models.Item>} subclass

    @ivar ignores_selection: Set to True for actions that don't operate on a
        selection of content.
    @type ignores_selection: bool

    @ivar min: The minimum number of content items that the action can be
        invoked on. Setting it to 0 or None disables the constraint.
    @type min: int

    @ivar max: The maximum number of content items that the action can be
        invoked on. A value of None disables the constraint.
    @type max: int

    @ivar direct_link: Set to True for actions that can provide a direct URL
        for their execution, without requiring a form submit and redirection.
    @type direct_link: bool

    @ivar parameters: A schema describing user supplied parameters required by
        the action. When set to None, the action requires no additional input
        from the user.
    @type parameters: L{Schema<cocktail.schema.schema.Schema>}
    """
    enabled = True
    included = frozenset(["toolbar_extra", "item_buttons_extra"])
    excluded = frozenset([
        "selector",
        "calendar_content_view",
        "workflow_graph_content_view",
        "changelog"
    ])
    content_type = None
    ignores_selection = False
    min = 1
    max = 1
    direct_link = False
    parameters = None

    def __init__(self, id):

        if not id:
            raise ValueError("User actions must have an id")

        if not isinstance(id, basestring):
            raise TypeError("User action identifiers must be strings, not %s"
                            % type(id))
        self._id = id

    def __translate__(self, language, **kwargs):
        return translations("Action " + self.id, **kwargs)

    @getter
    def id(self):
        """The unique identifier for the action.
        @type: str
        """
        return self._id

    def register(self, before = None, after = None):
        """Registers the action with the site, so that it can appear on action
        containers and be handled by controllers.

        Registering an action with an identifier already in use is allowed, and
        will override the previously registered action.

        @param before: Indicates the position for the registered action. Should
            match the identifier of an already registered action. The new
            action will be inserted immediately before the indicated action.
        @type before: str

        @param after: Indicates the position for the registered action. Should
            match the identifier of an already registered action. The new
            action will be inserted immediately after the indicated action.
        @type after: str

        @raise ValueError: Raised if both L{before} and L{after} are set.
        @raise ValueError: Raised if the position indicated by L{after} or
            L{before} can't be found.
        """
        if after and before:
            raise ValueError("Can't combine 'after' and 'before' parameters")

        prev_action = get_user_action(self._id)

        if after or before:
            if prev_action:
                _action_list._items.remove(prev_action)

            ref_action = _action_map[after or before]
            pos = _action_list.index(ref_action)

            if before:
                _action_list._items.insert(pos, self)
            else:
                _action_list._items.insert(pos + 1, self)

        elif prev_action:
            pos = _action_list.index(prev_action)
            _action_list._items[pos] = self
        else:
            _action_list._items.append(self)

        _action_map[self._id] = self

    def is_available(self, context, target):
        """Indicates if the user action is available under a certain context.

        @param context: A set of string identifiers, such as "context_menu",
            "toolbar", etc. Different views can make use of as many different
            identifiers as they require.
        @type container: str set

        @param target: The item or content type affected by the action.
        @type target: L{Item<woost.models.item.Item>} instance or class

        @return: True if the action can be shown in the given context, False
            otherwise.
        @rtype: bool
        """

        # Context filters
        def match(tokens):
            for token in tokens:
                if isinstance(token, str):
                    if token in context:
                        return True
                elif context.issuperset(token):
                    return True
            return False

        if self.included and not match(self.included):
            return False

        if self.excluded and match(self.excluded):
            return False

        # Content type filter
        if self.content_type is not None:
            if isinstance(target, type):
                if not issubclass(target, self.content_type):
                    return False
            else:
                if not isinstance(target, self.content_type):
                    return False

        # Authorization check
        return self.is_permitted(get_current_user(), target)

    def is_permitted(self, user, target):
        """Determines if the given user is allowed to execute the action.

        Subclasses should override this method in order to implement their
        access restrictions.

        @param user: The user to authorize.
        @type user: L{User<woost.models.user.User>}

        @param target: The item or content type affected by the action.
        @type target: L{Item<woost.models.item.Item>} instance or class

        @return: True if the user is granted permission, False otherwise.
        @rtype: bool
        """
        return True

    def get_dropdown_panel(self, target):
        """Produces the user interface fragment that should be shown as the
        content for the action's dropdown panel. Returning None indicates the
        action doesn't have a dropdown panel available.

        @param target: The item or content type affected by the action.
        @type target: L{Item<woost.models.item.Item>} instance or class

        @return: The user interface for the action's dropdown panel.
        @rtype: L{Element<cocktail.html.Element>}
        """
        return None

    def get_errors(self, controller, selection):
        """Validates the context of an action before it is invoked.

        @param controller: The controller that invokes the action.
        @type controller: L{Controller<cocktail.controllers.controller.Controller>}

        @param selection: The collection of items that the action will be
            applied to.
        @type selection: L{Item<woost.models.item.Item>} collection

        @return: The list of errors for the indicated context.
        @rtype: iterable L{Exception} sequence
        """
        if not self.ignores_selection:

            # Validate selection size
            selection_size = len(selection) if selection is not None else 0

            if (self.min and selection_size < self.min) \
            or (self.max is not None and selection_size > self.max):
                yield SelectionError(self, selection_size)

    def invoke(self, controller, selection):
        """Delegates control of the current request to the action. Actions can
        override this method to implement their own response logic; by default,
        users are redirected to an action specific controller.

        @param controller: The controller that invokes the action.
        @type controller: L{Controller<cocktail.controllers.controller.Controller>}
        """
        raise cherrypy.HTTPRedirect(self.get_url(controller, selection))

    def get_url(self, controller, selection):
        """Produces the URL of the controller that handles the action
        execution. This is used by the default implementation of the L{invoke}
        method. Actions can override this method to alter this value.

        By default, single selection actions produce an URL of the form
        $cms_base_url/content/$selected_item_id/$action_id. Other actions
        follow the form $cms_base_url/$action_id/?selection=$selection_ids

        @param controller: The controller that invokes the action.
        @type controller: L{Controller<cocktail.controllers.controller.Controller>}

        @param selection: The collection of items that the action is invoked
            on.
        @type selection: L{Item<woost.models.item.Item>} collection

        @return: The URL where the user should be redirected.
        @rtype: str
        """
        params = self.get_url_params(controller, selection)

        if self.ignores_selection:
            return controller.contextual_uri(self.id, **params)

        elif self.min == self.max == 1:
            # Single selection
            return controller.edit_uri(
                    selection[0],
                    self.id,
                    **params)
        else:
            return controller.contextual_uri(
                    self.id,
                    selection = [item.id for item in selection],
                    **params)

    def get_url_params(self, controller, selection):
        """Produces extra URL parameters for the L{get_url} method.

        @param controller: The controller that invokes the action.
        @type controller: L{Controller<cocktail.controllers.controller.Controller>}

        @param selection: The collection of items that the action is invoked
            on.
        @type selection: L{Item<woost.models.item.Item>} collection

        @return: A mapping containing additional parameters to include on the
            URL associated to the action.
        @rtype: dict
        """
        params = {}

        if controller.edit_stack:
            params["edit_stack"] = controller.edit_stack.to_param()

        return params

    @getter
    def icon_uri(self):
        return "/resources/images/%s_small.png" % self.id


class SelectionError(Exception):
    """An exception produced by the L{UserAction.get_errors} method when an
    action is attempted against an invalid number of items."""

    def __init__(self, action, selection_size):
        Exception.__init__(self, "Can't execute action '%s' on %d item(s)."
            % (action.id, selection_size))
        self.action = action
        self.selection_size = selection_size


# Implementation of concrete actions
#------------------------------------------------------------------------------

class CreateAction(UserAction):
    included = frozenset(["toolbar"])
    excluded = frozenset([
        "collection",
        ("selector", "existing_only"),
        "changelog"
    ])
    ignores_selection = True
    min = None
    max = None

    def get_url(self, controller, selection):
        return controller.edit_uri(controller.edited_content_type)


#class CreateBeforeAction(CreateAction):
#   ignores_selection = False


#class CreateInsideAction(CreateAction):
#   ignores_selection = False


#class CreateAfterAction(CreateAction):
#   ignores_selection = False


class MoveAction(UserAction):
    included = frozenset([("toolbar", "tree")])
    max = None


class AddAction(UserAction):
    included = frozenset([("toolbar", "collection")])
    excluded = frozenset(["integral"])
    ignores_selection = True
    min = None
    max = None

    def invoke(self, controller, selection):

        # Add a relation node to the edit stack, and redirect the user
        # there
        node = RelationNode()
        node.member = controller.member
        node.action = "add"
        controller.edit_stack.push(node)
        controller.edit_stack.go()


class AddIntegralAction(UserAction):

    included = frozenset([("collection", "toolbar", "integral")])
    ignores_selection = True
    min = None
    max = None

    def get_url(self, controller, selection):
        return controller.edit_uri(controller.root_content_type)


class RemoveAction(UserAction):
    included = frozenset([("toolbar", "collection")])
    excluded = frozenset(["integral"])
    max = None

    def invoke(self, controller, selection):

        stack_node = controller.stack_node

        for item in selection:
            stack_node.unrelate(controller.member, item)

        controller.user_collection.base_collection = \
            schema.get(stack_node.form_data, controller.member)


class OrderAction(UserAction):
    included = frozenset([("order_content_view", "toolbar")])
    max = None

    def invoke(self, controller, selection):
        node = RelationNode()
        node.member = controller.member
        node.action = "order"
        controller.edit_stack.push(node)
        UserAction.invoke(self, controller, selection)

    def get_url_params(self, controller, selection):
        params = UserAction.get_url_params(self, controller, selection)
        params["member"] = controller.section
        return params


class ShowDetailAction(UserAction):
    included = frozenset(["toolbar", "item_buttons"])


class EditAction(UserAction):
    included = frozenset(["toolbar", "item_buttons"])

    def is_permitted(self, user, target):
        return user.has_permission(ModifyPermission, target = target)

    def get_url(self, controller, selection):
        return controller.edit_uri(selection[0])


class DeleteAction(UserAction):
    included = frozenset([
        ("content", "toolbar"),
        ("collection", "toolbar", "integral"),
        "item_buttons"
    ])
    excluded = frozenset([
        "selector",
        "new_item",
        "calendar_content_view",
        "workflow_graph_content_view",
        "changelog"
    ])
    max = None

    def is_permitted(self, user, target):
        return user.has_permission(DeletePermission, target = target)


class ShowChangelogAction(UserAction):
    min = None
    max = 1
    excluded = frozenset([
        "selector",
        "new_item",
        "calendar_content_view",
        "workflow_graph_content_view",
        "changelog"
    ])

    def get_url(self, controller, selection):

        params = self.get_url_params(controller, selection)

        # Filter by target element
        if selection:
            params["filter"] = "member-changes"
            params["filter_value0"] = str(selection[0].id)

        # Filter by target type
        else:
            user_collection = getattr(controller, "user_collection", None)
            if user_collection and user_collection.type is not Item:
                params["filter"] = "target-type"
                params["filter_value0"] = user_collection.type.full_name

        return controller.contextual_uri(
            "changelog",
            **params
        )

    def is_permitted(self, user, target):
        return user.has_permission(ReadHistoryPermission)


class DiffAction(UserAction):
    included = frozenset(["item_buttons"])

    def is_permitted(self, user, target):
        return user.has_permission(ModifyPermission, target = target)


class RevertAction(UserAction):
    included = frozenset([("diff", "item_body_buttons", "changed")])

    def is_permitted(self, user, target):
        return user.has_permission(ModifyPermission, target = target)

    def invoke(self, controller, selection):

        reverted_members = controller.params.read(
            schema.Collection("reverted_members",
                type = set,
                items = schema.String
            )
        )

        stack_node = controller.stack_node

        form_data = stack_node.form_data
        form_schema = stack_node.form_schema
        languages = set(
            list(stack_node.translations)
            + list(stack_node.item.translations.keys())
        )

        source = {}
        stack_node.export_form_data(stack_node.item, source)

        for member in form_schema.members().itervalues():

            if member.translated:
                for lang in languages:
                    if (member.name + "-" + lang) in reverted_members:
                        schema.set(
                            form_data,
                            member.name,
                            schema.get(source, member.name, language = lang),
                            language = lang
                        )
            elif member.name in reverted_members:
                schema.set(
                    form_data,
                    member.name,
                    schema.get(source, member.name)
                )


class PreviewAction(UserAction):
    included = frozenset(["toolbar_extra", "item_buttons"])
    content_type = Publishable


class OpenResourceAction(UserAction):
    min = 1
    max = 1
    content_type = Publishable
    included = frozenset(["toolbar", "item_buttons"])
    excluded = frozenset([
        "new",
        "draft",
        "selector",
        "calendar_content_view",
        "workflow_graph_content_view",
        "changelog"
    ])

    def get_url(self, controller, selection):
        return controller.context["cms"].uri(selection[0])


class UploadFilesAction(UserAction):
    included = frozenset(["toolbar_extra"])
    content_type = File
    min = None
    max = None
    ignores_selection = True


class ExportAction(UserAction):
    included = frozenset(["toolbar_extra"])
    min = 1
    max = None
    ignores_selection = True
    format = None
    direct_link = True

    def __init__(self, id, format):
        UserAction.__init__(self, id)
        self.format = format

    def get_url(self, controller, selection):
        return "?" + view_state(
            format = self.format
        )


class SelectAction(UserAction):
    included = frozenset([("list_buttons", "selector")])
    excluded = frozenset()
    min = None
    max = None

    def invoke(self, controller, selection):

        stack = controller.edit_stack

        if stack:

            node = stack[-1]
            params = {}

            if isinstance(node, SelectionNode):
                params[node.selection_parameter] = (
                    selection[0].id
                    if selection
                    else ""
                )

            elif isinstance(node, RelationNode):
                edit_state = node.parent_node
                member = controller.stack_node.member

                if isinstance(member, schema.Reference):
                    edit_state.relate(
                        member,
                        None if not selection else selection[0]
                    )
                else:
                    if controller.stack_node.action == "add":
                        modify_relation = edit_state.relate
                    else:
                        modify_relation = edit_state.unrelate

                    for item in selection:
                        modify_relation(member, item)

            stack.go_back(**params)


class GoBackAction(UserAction):
    ignores_selection = True
    min = None
    max = None

    def invoke(self, controller, selection):
        controller.go_back()


class CloseAction(GoBackAction):
    included = frozenset(["item_buttons", ("changelog", "bottom_buttons")])
    excluded = frozenset(["changed", "new", "edit"])


class CancelAction(GoBackAction):
    included = frozenset([
        ("list_buttons", "selector"),
        ("item_buttons", "edit"),
        ("item_buttons", "changed"),
        ("item_buttons", "new"),
        ("item_bottom_buttons", "edit"),
        ("item_bottom_buttons", "changed"),
        ("item_bottom_buttons", "new")
    ])
    excluded = frozenset()


class SaveAction(UserAction):
    included = frozenset([
        ("item_buttons", "new"),
        ("item_buttons", "edit"),
        ("item_buttons", "changed"),
        ("item_bottom_buttons", "new"),
        ("item_bottom_buttons", "edit"),
        ("item_bottom_buttons", "changed")
    ])
    ignores_selection = True
    max = None
    min = None
    make_draft = False

    def is_permitted(self, user, target):
        if target.is_inserted:
            return user.has_permission(
                ModifyPermission,
                target = target
            )
        else:
            return user.has_permission(
                CreatePermission,
                target = target.__class__
            )

    def get_errors(self, controller, selection):
        for error in UserAction.get_errors(self, controller, selection):
            yield error

        for error in controller.stack_node.iter_errors():
            yield error

    def invoke(self, controller, selection):
        controller.save_item(make_draft = self.make_draft)


class SaveDraftAction(SaveAction):
    make_draft = True
    included = frozenset(["item_buttons", "item_bottom_buttons"])
    excluded = frozenset(["draft"])

    def is_permitted(self, user, target):
        return True

class ConfirmDraftAction(SaveAction):
    confirm_draft = True
    included = frozenset([
        ("item_buttons", "draft"),
        ("item_bottom_buttons", "draft")
    ])
    excluded = frozenset()

    def is_permitted(self, user, target):
        return user.has_permission(ConfirmDraftPermission, target = target)

    def invoke(self, controller, selection):
        controller.confirm_draft()


class PrintAction(UserAction):
    direct_link = True
    ignore_selection = True
    excluded = frozenset(["selector"])

    def get_url(self, controller, selection):
        return "javascript: print();"


# Action registration
#------------------------------------------------------------------------------
CreateAction("new").register()
MoveAction("move").register()
AddAction("add").register()
AddIntegralAction("add_integral").register()
RemoveAction("remove").register()
ShowDetailAction("show_detail").register()
PreviewAction("preview").register()
OpenResourceAction("open_resource").register()
UploadFilesAction("upload_files").register()
EditAction("edit").register()
DiffAction("diff").register()
RevertAction("revert").register()
ShowChangelogAction("changelog").register()
DeleteAction("delete").register()
OrderAction("order").register()
ExportAction("export_xls", "msexcel").register()
PrintAction("print").register()
CloseAction("close").register()
CancelAction("cancel").register()
SaveAction("save").register()
SaveDraftAction("save_draft").register()
ConfirmDraftAction("confirm_draft").register()
SelectAction("select").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.controllers import (
    request_property,
    Form
)
from woost.controllers.notifications import pop_user_notifications
from woost.extensions.ecommerce.catalogcontroller \
    import catalog_current_state_uri


class OrderStepForm(Form):

    @request_property
    def order_steps(self):
        from woost.extensions.ecommerce import ECommerceExtension
        return ECommerceExtension.instance.order_steps

    @request_property
    def current_step(self):
        return self.controller.context["publishable"]

    @request_property
    def next_step(self):
        steps = self.order_steps
        pos = steps.index(self.current_step)
        if pos + 1 < len(steps):
            return steps[pos + 1]

    @request_property
    def previous_step(self):
        steps = self.order_steps
        pos = steps.index(self.current_step)
        if pos > 0:
            return steps[pos - 1]

    def proceed(self):
        pop_user_notifications()
        next_step = self.next_step
        if next_step is not None:
            raise cherrypy.HTTPRedirect(next_step.get_uri())

    def back(self):
        pop_user_notifications()
        previous_step = self.previous_step
        if previous_step is None:
            raise cherrypy.HTTPRedirect(catalog_current_state_uri())
        else:
            raise cherrypy.HTTPRedirect(previous_step.get_uri())


class ProceedForm(OrderStepForm):

    actions = "proceed",

    def submit(self):
        self.proceed()


class BackForm(OrderStepForm):

    actions = "back",

    def submit(self):
        self.back()




#-*- coding: utf-8 -*-
u"""

@author:        Jordi Fernández
@contact:       jordi.fernandez@whads.com
@organization:  Whads/Accent SL
@since:         March 2010
"""
from cocktail.translations import translations

translations.define(
    "woost.extensions.campaignmonitor.subscription_controller.title",
    ca = u"Controlador de subscripció a Campaign Monitor",
    es = u"Controlador de suscripicón a Campaign Monitor",
    en = u"Subscription to Campaign Monitor controller"
)

translations.define(
    "woost.extensions.campaignmonitor.unsubscription_controller.title",
    ca = u"Controlador de baixa de Campaign Monitor",
    es = u"Controlador de baja de Campaign Monitor",
    en = u"Unsubscription to Campaign Monitor controller"
)

translations.define(
    "woost.extensions.campaignmonitor.subscription_template.title",
    ca = u"Vista de subscripció a Campaign Monitor",
    en = u"Subscription to Campaign Monitor view",
    es = u"Vista de suscripción a Campaign Monitor"
)

translations.define(
    "woost.extensions.campaignmonitor.unsubscription_template.title",
    ca = u"Plantilla de baixa de Campaign Monitor",
    es = u"Plantilla de baja de Campaign Monitor",
    en = u"Unsubscription from Campaign Monitor template"
)

translations.define(
    "woost.extensions.campaignmonitor.subscription_page.title",
    ca = u"Pàgina de subscripció",
    es = u"Página de suscripción",
    en = u"Subscription page"
)

translations.define(
    "woost.extensions.campaignmonitor.pending_page.title",
    ca = u"Pàgina d'instruccions de subscripció",
    es = u"Página de instrucciones de suscripcion",
    en = u"Page with instructions for subscription"
)

translations.define(
    "woost.extensions.campaignmonitor.pending_page.body",
    ca = u"T'hem enviat un e-mail per confirmar la teva adreça de correu "
         u"electrònic. Si et plau, fes clic a l'enllaç que apareix a "
         u"l'e-mail per confirmar la teva subscripció.",
    es = u"Te hemos enviado un e-mail para confirmar tu dirección de correo "
         u"electrónico. Por favor, haz clic en el enlace que aparece en el "
         u"e-mail para confirmar tu suscripción.""",
    en = u"We've just been sent an email to confirm your email address. "
         u"Please click on the link in this email to confirm your "
         u"subscription."
)

translations.define(
    "woost.extensions.campaignmonitor.confirmation_success_page.title",
    ca = u"Pàgina de confirmació",
    es = u"Página de confirmación",
    en = u"Confirmation page"
)

translations.define(
    "woost.extensions.campaignmonitor.confirmation_success_page.body",
    ca = u"La teva subscripció ha estat confirmada. Hem afegit la teva "
         u"adreça a la nostra llista i en breu rebràs notícies nostres.",
    es = u"Tu suscripción ha sido confirmada. Se ha añadido tu dirección a "
         u"nuestra lista y en breve recibirás noticias nuestras.",
    en = u"Your subscription has been confirmed. You've been added to our "
         u"list and will hear from us soon."
)

translations.define(
    "woost.extensions.campaignmonitor.unsubscribe_page.title",
    ca = u"Pàgina de baixa",
    es = u"Página de baja",
    en = u"Unsubscribe page"
)

translations.define(
    "woost.extensions.campaignmonitor.unsubscribe_page.body",
    ca = u"La teva adreça s'ha retirat amb èxit d'aquesta llista de "
         u"subscripció. No tornaràs a rebre'n missatges.",
    es = u"Tu dirección ha sido eliminada con éxito de esta lista de "
         u"suscripción. No volverás a recibir mensajes de esta lista.",
    en = u"You have been successfully removed from this subscriber list. You "
         u"will no longer receive messages from this list."
)

# UI
#------------------------------------------------------------------------------
translations.define("Action sync_campaign_monitor_lists",
    ca = u"Sincronitzar amb Campaign Monitor",
    es = u"Sincronizar con Campaign Monitor",
    en = u"Synchronize with Campaign Monitor"
)

# CampaignMonitorExtension
#------------------------------------------------------------------------------
translations.define("CampaignMonitorExtension.api_key",
    ca = u"Clau d'API",
    es = u"Clave de API",
    en = u"API key"
)

translations.define("CampaignMonitorExtension.client_id",
    ca = u"Id de client",
    es = u"Id de cliente",
    en = u"Client id"
)

translations.define("CampaignMonitorExtension.lists",
    ca = u"Llistes",
    es = u"Listas",
    en = u"Lists"
)

# CampaignMonitorList
#------------------------------------------------------------------------------
translations.define("CampaignMonitorList",
    ca = u"Llista de subscripció",
    es = u"Lista de suscripción",
    en = u"Subscription list"
)

translations.define("CampaignMonitorList-plural",
    ca = u"Llistes de subscripció",
    es = u"Listas de suscripción",
    en = u"Subscription lists"
)

translations.define("CampaignMonitorList.list_id",
    ca = u"Id de llista",
    es = u"Id de lista",
    en = u"List id"
)

translations.define("CampaignMonitorList.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("CampaignMonitorList.pending_page",
    ca = u"Pàgina d'instruccions",
    es = u"Página de instrucciones",
    en = u"Instructions page"
)

translations.define("CampaignMonitorList.unsubscribe_page",
    ca = u"Pàgina de baixa",
    es = u"Página de baja",
    en = u"Unsubscribe page"
)

translations.define("CampaignMonitorList.confirmation_success_page",
    ca = u"Pàgina de confirmació d'alta",
    es = u"Página de confirmación de alta",
    en = u"Confirmation success page"
)

# CampaignMonitorSubscriptionPage
#------------------------------------------------------------------------------
translations.define("CampaignMonitorSubscriptionPage",
    ca = u"Pàgina de subscripció a mailing",
    es = u"Página de suscripción a mailing",
    en = u"Mailing subscription page"
)

translations.define("CampaignMonitorSubscriptionPage-plural",
    ca = u"Pàgines de subscripció a mailing",
    es = u"Páginas de suscripción a mailing",
    en = u"Mailing subscription pages"
)

translations.define("CampaignMonitorSubscriptionPage.body",
    ca = u"Cos",
    es = u"Cuerpo",
    en = u"Body"
)

translations.define("CampaignMonitorSubscriptionPage.lists",
    ca = u"Llistes",
    es = u"Listas",
    en = u"Lists"
)

translations.define("CampaignMonitorSubscriptionPage.subscription_form",
    ca = u"Formulari de subscripció",
    es = u"Formulario de suscripción",
    en = u"Subscription form"
)

# CampaignMonitorListsSynchronizationView
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView outcome",
    ca = u"S'ha completat la sincronització.",
    es = u"Se ha completado la sincronización.",
    en = u"Synchronization complete."
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView no changes",
    ca = u"No ha estat necessari realitzar cap actualització.",
    es = u"No ha sido necesario realizar ninguna actualización.",
    en = u"No update was needed."
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView cancel button",
    ca = u"Tancar",
    es = u"Cerrar",
    en = u"Close"
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView created lists title",
    ca = u"Llistes afegides",
    es = u"Listas añadidas",
    en = u"Added lists"
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView created lists explanation",
    ca = u"Les següents llistes s'han incorporat al lloc web:",
    es = u"Las listas siguientes se han incorporado al sitio web:",
    en = u"The following lists were added to the web site:"
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView modified lists title",
    ca = u"Llistes modificades",
    es = u"Listas modificadas",
    en = u"Modified lists"
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView modified lists explanation",
    ca = u"S'ha actualitzat la informació de les llistes següents:",
    es = u"Se ha actualizado la información de las listas siguientes:",
    en = u"The following lists had changes that have been reflected on the "
         u"web site:"
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView deleted lists title",
    ca = u"Llistes eliminades",
    es = u"Listas eliminadas",
    en = u"Deleted lists"
)

translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView deleted lists explanation",
    ca = u"Les següents llistes ja no són presents a Campaign Monitor i s'han "
         u"eliminat del lloc web:",
    es = u"Las listas siguientes ya no están disponibles en Campaign Monitor y se "
         u"han eliminado del sitio web:",
    en = u"The following lists are no longer available at Campaign Monitor and "
         u"have been deleted from the web site:"
)

# CampaignMonitorSubscriptionView
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorSubscriptionView api error",
    ca = u"Hi ha hagut algun problema. Siusplau, provi-ho de nou més tard.",
    es = u"Ha habido algun problema, por favor pruebelo más tarde.",
    en = u"An error occurred, please try again later."
)

# CampaignMonitorUnsubscriptionView
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.campaignmonitor.CampaignMonitorUnsubscriptionView resubscribe",
    ca = u"""No volies donar-te de baixa? <a href="%s">Clica aquí</a> per
resubscriure't i continuar rebent comunicacions nostres.""",
    es = u"""No querías darte de baja? <a href="%s">Clica aquí</a> para
resuscribirte y continuar recibiendo comunicaciones nuestras.""",
    en = u"""Didn't mean to unsubscribe? <a href="%s">Click here</a> to
re-subscribe and continue receiving emails from us."""
)

# SubscriptionForm
#------------------------------------------------------------------------------
translations.define("SubscriptionForm.name",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("SubscriptionForm.email",
    ca = u"Correu electrònic",
    es = u"Correo electrónico",
    en = u"E-mail address"
)

translations.define("SubscriptionForm.list",
    ca = u"Llista",
    es = u"Lista",
    en = u"List"
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.controllers.renderingengines import get_rendering_engine
from woost.controllers.publishablecontroller import PublishableController


class DocumentController(PublishableController):
    """A controller that serves rendered pages."""

    @cached_getter
    def page_template(self):
        template = self.context["publishable"].template

        if template is None:
            raise cherrypy.NotFound()

        return template

    @cached_getter
    def rendering_engine(self):
        engine_name = self.page_template.engine

        if engine_name:
            return get_rendering_engine(
                engine_name,
                cherrypy.request.config.get("rendering.engine_options")
            )
        else:
            return PublishableController.rendering_engine(self)

    @cached_getter
    def view_class(self):
        return self.page_template.identifier




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from cocktail.translations import translations
from woost.models import Extension

translations.define("WorkflowExtension",
    ca = u"Circuïts de treball",
    es = u"Circuitos de trabajo",
    en = u"Workflow"
)

translations.define("WorkflowExtension-plural",
    ca = u"Circuïts de treball",
    es = u"Circuitos de trabajo",
    en = u"Workflow"
)


class WorkflowExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet construïr circuïts de treball, mitjançant la definició
            d'estats i transicions pels elements.""",
            "ca"
        )
        self.set("description",
            u"""Permite construir circuitos de trabajo, mediante la definición
            de estados y transiciones para los elementos.""",
            "es",
        )
        self.set("description",
            u"""Allows the set up of workflows, based on a system of item
            states and transitions..""",
            "en"
        )

    def _load(self):

        from woost.extensions.workflow import (
            strings,
            state,
            transition,
            transitionpermission,
            transitiontrigger,
            setstatetriggerresponse,
            useractions
        )

        # Add a controller to handle transition schemas
        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController
        from woost.extensions.workflow.transitioncontroller \
            import TransitionController

        BackOfficeController.workflow_transition = TransitionController

        # Add a content view for workflow graphs (if GraphViz is installed)
        from woost.extensions.workflow.graph import graphviz_command

        if graphviz_command:

            from woost.controllers.backoffice.contentviews \
                import global_content_views

            global_content_views.add(
                state.State,
                "woost.extensions.workflow.WorkflowGraphContentView",
                is_default = True
            )

            # Add a controller to render the SVG graphs for the graphs content view
            from woost.controllers.backoffice.backofficecontroller \
                import BackOfficeController
            from woost.extensions.workflow.workflowgraphcontroller \
                import WorkflowGraphController

            BackOfficeController.workflow_graph = WorkflowGraphController




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail.translations import translations
from cocktail import schema
from woost.models import (
    Item,
    get_current_user,
    ModifyMemberPermission
)
from woost.extensions.ecommerce.ecommercebillingconcept \
    import ECommerceBillingConcept


class ECommercePurchase(Item):

    listed_from_root = False

    members_order = [
        "order",
        "product",
        "quantity",
        "total_price",
        "pricing",
        "total_shipping_costs",
        "shipping_costs",
        "total_taxes",
        "taxes",
        "total"
    ]

    order = schema.Reference(
        type = "woost.extensions.ecommerce.ecommerceorder.ECommerceOrder",
        bidirectional = True,
        required = True
    )

    product = schema.Reference(
        type = "woost.extensions.ecommerce.ecommerceproduct.ECommerceProduct",
        bidirectional = True,
        required = True
    )

    quantity = schema.Integer(
        required = True,
        min = 1,
        default = 1
    )

    total_price = schema.Decimal(
        member_group = "billing",
        editable = False,
        listed_by_default = False
    )

    pricing = schema.Collection(
        member_group = "billing",
        items = schema.Reference(type = ECommerceBillingConcept),
        related_end = schema.Collection(
            block_delete = True
        ),
        editable = False
    )

    total_shipping_costs = schema.Decimal(
        member_group = "billing",
        editable = False,
        listed_by_default = False
    )

    shipping_costs = schema.Collection(
        member_group = "billing",
        items = schema.Reference(type = ECommerceBillingConcept),
        related_end = schema.Collection(
            block_delete = True
        ),
        editable = False
    )

    total_taxes = schema.Decimal(
        member_group = "billing",
        editable = False,
        listed_by_default = False
    )

    taxes = schema.Collection(
        member_group = "billing",
        items = schema.Reference(type = ECommerceBillingConcept),
        related_end = schema.Collection(
            block_delete = True
        ),
        editable = False
    )

    total = schema.Decimal(
        member_group = "billing",
        editable = False
    )

    def __translate__(self, language, **kwargs):
        if self.draft_source is not None:
            return Item.__translate__(self, language, **kwargs)

        desc = u"%d x %s" % (
            self.quantity,
            translations(self.product, language)
        )

        options = []
        for member in self.get_options():
            if member is ECommercePurchase.quantity:
                continue
            options.append("%s: %s" % (
                translations(member, language),
                member.translate_value(self.get(member), language)
            ))

        if options:
            desc += u" (%s)" % u", ".join(options)

        return desc

    def calculate_costs(self,
        apply_pricing = True,
        apply_shipping_costs = True,
        apply_taxes = True
    ):
        from woost.extensions.ecommerce import ECommerceExtension
        extension = ECommerceExtension.instance

        purchase_costs = {
            "price": {
                "cost": self.get_unit_price(),
                "paid_units": self.quantity,
                "percentage": Decimal("0.00"),
                "concepts": []
            },
            "shipping_costs": {
                "cost": Decimal("0.00"),
                "paid_units": self.quantity,
                "percentage": Decimal("0.00"),
                "concepts": []
            },
            "taxes": {
                "cost": Decimal("0.00"),
                "paid_units": self.quantity,
                "percentage": Decimal("0.00"),
                "concepts": []
            }
        }

        # Price
        purchase_price = purchase_costs["price"]

        if apply_pricing:
            for pricing in extension.pricing:
                if pricing.applies_to(self, purchase_costs):
                    pricing.apply(self, purchase_price)

        purchase_price["cost"] += \
            purchase_price["cost"] * purchase_price["percentage"] / 100

        purchase_price["total"] = \
            purchase_price["cost"] * purchase_price["paid_units"]

        # Shipping costs
        purchase_shipping_costs = purchase_costs["shipping_costs"]

        if apply_shipping_costs:
            for shipping_cost in extension.shipping_costs:
                if shipping_cost.applies_to(self, purchase_costs):
                    shipping_cost.apply(self, purchase_shipping_costs)

        purchase_shipping_costs["cost"] += \
            purchase_price["cost"] * purchase_shipping_costs["percentage"] / 100

        purchase_shipping_costs["total"] = \
            purchase_shipping_costs["cost"] * purchase_shipping_costs["paid_units"]

        # Taxes
        purchase_taxes = purchase_costs["taxes"]

        if apply_taxes:
            for tax in extension.taxes:
                if tax.applies_to(self, purchase_costs):
                    tax.apply(self, purchase_taxes)

        purchase_taxes["cost"] += \
            purchase_price["cost"] * purchase_taxes["percentage"] / 100

        purchase_taxes["total"] = \
            purchase_taxes["cost"] * purchase_taxes["paid_units"]

        # Total
        purchase_costs["total"] = (
            purchase_price["total"]
          + purchase_shipping_costs["total"]
          + purchase_taxes["total"]
        )
        return purchase_costs

    def get_unit_price(self):
        return self.product.price

    def get_weight(self):
        if self.product is None or self.product.weight is None:
            return 0
        else:
            return self.quantity * self.product.weight

    @classmethod
    def get_options(cls):
        for member in cls.members().itervalues():
            if (
                member is not cls.product
                and member is not cls.order
                and member.visible
                and member.editable
                and issubclass(member.schema, ECommercePurchase)
                and get_current_user().has_permission(
                    ModifyMemberPermission,
                    member = member
                )
            ):
                yield member




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from cocktail.tests.seleniumtester import selenium_test, browser
from woost.tests.test_selenium import (
    admin_email,
    admin_password
)


class AuthenticationTestCase(object):

    def assert_login_form_visible(self):
        assert browser.is_text_present("User")
        assert browser.is_text_present("Password")
        assert browser.is_element_present("xpath=//input[@name='user']")
        assert browser.is_element_present(
            "xpath=//input[@name='password' and @type='password']"
        )

    def login(self, user_name = "", password = ""):
        browser.type("user", user_name)
        browser.type("password", password)
        browser.click("authenticate")
        browser.wait_for_page_to_load(10000)

    @selenium_test
    def test_unauthorized_access_shows_login_form(self):
        browser.open("/en/cms")
        self.assert_login_form_visible()

    @selenium_test
    def test_wrong_login_attempt_fails(self):
        browser.open("/en/cms")

        self.login()
        self.assert_login_form_visible()

        self.login(admin_email + "asdk23h4")
        self.assert_login_form_visible()

        self.login(admin_email)
        self.assert_login_form_visible()

        self.login(admin_email, admin_password + "kn324n1")
        self.assert_login_form_visible()

    @selenium_test
    def test_valid_login_attempt_succeeds(self):
        browser.open("/en/cms")
        self.login(admin_email, admin_password)
        assert browser.is_text_present(admin_email)
        assert browser.is_element_present("css=.logout_button")

    @selenium_test
    def test_logout(self):
        browser.open("/en/cms")
        self.login(admin_email, admin_password)
        browser.click("css=.logout_button")
        browser.wait_for_page_to_load(5000)
        self.assert_login_form_visible()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.html import templates
from woost.models import get_current_user, ReadPermission

List = templates.get_class("cocktail.html.List")


class ContentList(List):

    referer = None

    def _fill_entries(self):

        user = get_current_user()
        items = self.items

        if items is not None:
            items = [
                item
                for item in self.items
                if user.has_permission(ReadPermission, target = item)
            ]

        if items:
            List._fill_entries(self)
        else:
            self.tag = "div"
            self.append(u"-")

    def create_entry_content(self, item):
        link = templates.new("woost.views.ContentLink")
        link.item = item
        link.referer = self.referer
        link.member = self.member
        return link




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from threading import Lock
import cherrypy
from cocktail.modeling import cached_getter, getter
from cocktail.events import event_handler
from cocktail.pkgutils import resolve
from cocktail import schema
from cocktail.controllers import view_state, Location, get_parameter
from woost.models import (
    Item,
    ReadPermission,
    get_current_user
)

from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController

from woost.controllers.backoffice.editstack import EditNode, RelationNode

from woost.controllers.backoffice.showdetailcontroller \
    import ShowDetailController

from woost.controllers.backoffice.differencescontroller \
    import DifferencesController


class ItemController(BaseBackOfficeController):

    default_section = "fields"

    diff = DifferencesController

    @cached_getter
    def preview(self):
        return resolve(self.stack_node.item.preview_controller)

    @cached_getter
    def show_detail(self):
        return resolve(self.stack_node.item.show_detail_controller)

    @cached_getter
    def fields(self):
        return resolve(self.stack_node.item.edit_controller)

    def resolve(self, path):

        if path:
            collection_name = path.pop(0)

            try:
                member = self.stack_node.content_type[collection_name]
            except KeyError:
                pass
            else:
                if any(
                    collection.name == member.name
                    for collection in self.collections
                ):
                    return self._get_collection_controller(member)

    def _get_collection_controller(self, member):
        controller_class = resolve(member.edit_controller)
        return controller_class(member)

    @cached_getter
    def collections(self):

        relation_node = self.relation_node
        stack_relation = relation_node and relation_node.member.related_end

        return [
            member
            for member in self.stack_node.form_schema.ordered_members()
            if isinstance(member, schema.Collection)
            and not member.edit_inline
        ]

    @event_handler
    def handle_traversed(cls, event):

        controller = event.source

        # Require an edit stack with an edit node on top
        controller._require_edit_node()

        # Disable access to invisible content types
        if not controller.stack_node.content_type.visible:
            raise cherrypy.NotFound()

        # Restrict access
        if controller.stack_node.item.is_inserted:
            get_current_user().require_permission(
                ReadPermission,
                target = controller.stack_node.item
            )

    def _require_edit_node(self):

        redirect = False
        context_item = self.context["cms_item"]
        edit_stacks_manager = self.context["edit_stacks_manager"]
        edit_stack = edit_stacks_manager.current_edit_stack

        # Spawn a new edit stack
        if edit_stack is None:
            edit_stack = edit_stacks_manager.create_edit_stack()
            edit_stacks_manager.current_edit_stack = edit_stack
            redirect = True
        else:
            # Integral part; add a new relation node (won't be shown to the
            # user)
            member_name = self.params.read(schema.String("member"))

            if member_name:
                node = RelationNode()
                node.member = edit_stack[-1].content_type[member_name]
                edit_stack.push(node)
                redirect = True

        # Make sure the top node of the stack is an edit node
        if not edit_stack \
        or not isinstance(edit_stack[-1], EditNode) \
        or (context_item and context_item.id != edit_stack[-1].item.id):

            # New item
            if context_item is None:
                content_type = get_parameter(
                    schema.Reference("item_type", class_family = Item)
                )
                item = content_type()
            # Existing item
            else:
                item = context_item

            node_class = resolve(item.edit_node_class)
            node = node_class(item)
            edit_stack.push(node)
            redirect = True

            if not item.is_inserted:
                node.initialize_new_item(
                    item,
                    get_current_user(),
                    self.visible_languages
                )

        # If the stack is modified a redirection is triggered so that any
        # further request mentions the new stack position in its parameters.
        # However, the redirection won't occur if the controller itself is the
        # final target of the current request - if that is the case, submit()
        # will end up redirecting the user to the default section anyway
        if redirect and self is not cherrypy.request.handler:
            location = Location.get_current()
            location.method = "GET"
            location.params["edit_stack"] = edit_stack.to_param()
            location.params.pop("member", None)
            location.go()

        return edit_stack

    def submit(self):
        self.section_redirection(self.default_section)

    def section_redirection(self, default = None):
        section = cherrypy.request.params.get("section", default)
        if section:
            self.switch_section(section)

    def switch_section(self, section):
        item = self.stack_node.item

        # Preserve form initialization parameters
        if cherrypy.request.method == "GET":
            params = dict(
                (key, value)
                for key, value in cherrypy.request.params.iteritems()
                if key.startswith("edited_item_")
            )
        else:
            params = {}

        raise cherrypy.HTTPRedirect(
            self.edit_uri(
                item if item.is_inserted else item.__class__,
                section,
                **params
            )
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.rendering.effects import image_effects

def apply_effects_chain(image, effects_chain):

    for func, args, kwargs in normalize_effects_chain(effects_chain):
        image = func(image, *args, **kwargs)

    return image

def normalize_effects_chain(effects_chain):

    if not effects_chain:
        return []

    if isinstance(effects_chain, basestring):
        effects_chain = effects_chain.split("/")

    return [
        parse_effect(effect) if isinstance(effect, basestring) else effect
        for effect
        in effects_chain
    ]

def parse_effect(effects_string):

    pos = effects_string.find("(")
    if pos == -1:
        effects_name = effects_string
    else:
        effects_name = effects_string[:pos]

    try:
        func = image_effects[effects_name]
    except KeyError:
        raise ValueError("Unknown image effect: %s" % effects_name)

    args = []
    kwargs = {}

    if pos != -1:
        if effects_string[-1] != ")":
            raise ValueError(
                "Invalid image effect string: %r" % effects_string
            )

        for arg in effects_string[pos + 1:-1].split(","):
            parts = arg.split("=")

            # Positional parameters
            if len(parts) == 1:
                value = _parse_value(arg)
                args.append(value)

            # Keyword parameters
            elif len(parts) == 2:
                key, value = parts
                value = _parse_value(value)
                kwargs[key] = value
            else:
                raise ValueError(
                    "Invalid image effects string: %r" % effects_string
                )

    return func, args, kwargs

def _parse_value(value):

    if len(value) >= 2 and value[0] == value[-1] and value[0] in ("'\""):
        return value[1:-1]

    if value in ("True", "False"):
        return bool(value)

    if "." in value:
        try:
            value = float(value)
        except ValueError:
            pass
        else:
            return value

    try:
        value = int(value)
    except ValueError:
        pass
    else:
        return value

    raise ValueError(
        "Invalid value for image effect parameter: %r" % value
    )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from cocktail.events import when
from cocktail import schema
from woost.models import Item, Action, ChangeSet
from woost.models.trigger import (
    Trigger,
    actions_with_triggers,
    _handles_action,
    trigger_responses
)
from woost.extensions.workflow.state import State

actions_with_triggers.add("transition")

Trigger.add_member(
    schema.Collection(
        "item_states",
        items = schema.Reference(type = State, required = True),
        related_end = schema.Collection(),
        edit_inline = True
    )
)

Trigger.add_member(
    schema.Collection(
        "item_previous_states",
        items = schema.Reference(type = State, required = True),
        related_end = schema.Collection(),
        #exclusive = _handles_action("transition"),
        edit_inline = True
    )
)

_base_matches = Trigger.matches

def _matches(self, item, action, user, **context):

    if not _base_matches(self, item, action, user, **context):
        return False

    # Check current state
    if self.item_states and item.state not in self.item_states:
        return False

    # Check source state
    if self.item_previous_states:
        previous_state = context.get("previous_state")
        if previous_state is None \
        or previous_state not in self.item_previous_states:
            return False

    return True

Trigger.matches = _matches

@when(Item.changed)
def _trigger_transition_responses(event):
    if event.member is Item.state:
        trigger_responses(
            event.source,
            Action.get_instance(identifier = "transition"),
            ChangeSet.current_author,
            previous_state = event.previous_value
        )




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""
import os
from shutil import rmtree
from subprocess import Popen, PIPE
from cocktail import schema
from cocktail.controllers import context as controller_context
from cocktail.modeling import abstractmethod, getter
from cocktail.translations import translations
from cocktail.controllers.location import Location
from woost import app
from woost.models import Item


class StaticSiteSnapShoter(Item):
    """A class tasked with creating a static snapshot of a site's content to
    a concrete location.

    This is mostly an abstract class, meant to be extended by subclasses. Each
    subclass should implement the static snapshot creation.
    """
    instantiable = False
    visible_from_root = False
    integral = True

    def setup(self, context):
        """Prepares the exporter for an export process.

        The object of this method is to allow exporters to perform any
        initialization they require before writing files to their destination.

        @param context: A dictionary where the exporter can place any
            contextual information it many need throgout the export process. It
            will be made available to all L{write_file} calls.
        @type context: dict
        """

    def cleanup(self, context):
        """Frees resources after an export operation.

        This method is guaranteed to be called after the export operation is
        over, even if an exception was raised during its execution.

        @param context: The dictionary used by the exporter during the export
            process to maintain its contextual information.
        @type context: dict
        """

    def snapshot(self, items, context = {}):
        """ Generates the snapshot of a site's content

        @param items: The list of items which the exportation will start.
        @type items: L{Publishable}

        @param context: A dictionary used to share any contextual information
            with the snapshoter.
        @type context: dict
        """
        self.setup(context)

        try:
            for item in items:
                for file_data in self._snapshot(
                    item,
                    context = context
                ):
                    yield file_data
        finally:
            self.cleanup(context)

    @abstractmethod
    def _snapshot(self, root, context = {}):
        """ Generates the snapshot of a site's content

        @param root: The item which the exportation will statrt.
        @type root: Publishable

        @param context: A dictionary used to share any contextual information
            with the snapshoter.
        @type context: dict
        """

class WgetSnapShoter(StaticSiteSnapShoter):
    """ A class that creates a static snapshot of a site's content using wget """
    instantiable = True

    file_names_mode = schema.String(
        required = True,
        default = "unix",
        text_search = False,
        enumeration = frozenset(("unix", "windows")),
        translate_value = lambda value, **kwargs:
            u"" if not value else translations(
                "woost.extensions.staticsite.staticsitesnapshoter.WgetSnapShoter.file_names_mode " + value,
                **kwargs
            )
    )

    @getter
    def snapshot_path(self):
        return app.path("snapshots", str(self.id))

    def _get_cmd(self, context):
        cmd = "wget --mirror"

        if not context.get("follow_links"):
            cmd += " --level 1"

        cmd += " --page-requisites --html-extension \
                --convert-links --no-host-directories \
                --no-check-certificate --directory-prefix=%s \
                --restrict-file-names=%s %s"

        return cmd

    def _snapshot(self, root, context = {}):

        cmd = self._get_cmd(context)

        uri = self._get_uri(root, context)

        cmd = cmd % (
            self.snapshot_path,
            self.file_names_mode,
            uri
        )

        p = Popen(cmd, shell=True, stdout=PIPE)
        p.communicate()

        for root, dirs, files in os.walk(self.snapshot_path):
            for file in files:
                file_path = os.path.join(root, file).decode("utf-8")
                relative_path = os.path.relpath(file_path, self.snapshot_path)
                yield (file_path, relative_path)

    def _get_uri(self, item, context):
        location = Location.get_current_host()
        location.path_info = controller_context["cms"].uri(item)

        return unicode(location).encode("utf-8")

    def cleanup(self, context):
        if os.path.exists(self.snapshot_path):
            rmtree(self.snapshot_path)






#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Location
#------------------------------------------------------------------------------
translations.define("Location",
    ca = u"Ubicació",
    es = u"Ubicación",
    en = u"Location"
)

translations.define("Location-plural",
    ca = u"Ubicacions",
    es = u"Ubicaciones",
    en = u"Locations"
)

translations.define("Location.location_name",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("Location.location_type",
    ca = u"Tipus d'ubicació",
    es = u"Tipo de ubicación",
    en = u"Location type"
)

translations.define("woost.extensions.locations.location_types.continent",
    ca = u"Continent",
    es = u"Continente",
    en = u"Continent"
)

translations.define("woost.extensions.locations.location_types.country",
    ca = u"País",
    es = u"País",
    en = u"Country"
)

translations.define(
    "woost.extensions.locations.location_types.autonomous_community",
    ca = u"Comunitat autònoma",
    es = u"Comunidad autónoma",
    en = u"Autonomous community"
)

translations.define(
    "woost.extensions.locations.location_types.province",
    ca = u"Província",
    es = u"Provincia",
    en = u"Province"
)

translations.define(
    "woost.extensions.locations.location_types.town",
    ca = u"Població",
    es = u"Población",
    en = u"Town"
)

translations.define("Location.code",
    ca = u"Codi",
    es = u"Código",
    en = u"Code"
)

translations.define("Location.parent",
    ca = u"Ubicació pare",
    es = u"Ubicación padre",
    en = u"Parent location"
)

translations.define("Location.locations",
    ca = u"Ubicacions contingudes",
    es = u"Ubicaciones contenidas",
    en = u"Child locations"
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from decimal import Decimal
from cocktail.translations import translations
from cocktail import schema
from woost.models import Publishable, URI

Publishable.add_member(
    schema.Boolean(
        "sitemap_indexable",
        required = True,
        default = True,
        indexed = True,
        member_group = "sitemap",
        listed_by_default = False
    ),
    append = True
)

URI.default_sitemap_indexable = False

Publishable.add_member(
    schema.String("sitemap_change_frequency",
        enumeration = [
            "always",
            "hourly",
            "daily",
            "weekly",
            "monthly",
            "yearly",
            "never"
        ],
        translate_value = lambda value, language = None, **kwargs:
            "" if not value
            else translations(
                "woost.extensions.sitemap.change_frequency " + value,
                language,
                **kwargs
            ),
        member_group = "sitemap",
        text_search = False,
        listed_by_default = False
    ),
    append = True
)

Publishable.add_member(
    schema.Decimal("sitemap_priority",
        default = Decimal("0.5"),
        min = 0,
        max = 1,
        listed_by_default = False,
        member_group = "sitemap"
    ),
    append = True
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import os.path
from string import ascii_letters
from sha import sha
from random import choice
from warnings import warn

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import rfc822
import cherrypy
from cherrypy.lib.cptools import validate_since
from pkg_resources import resource_filename, iter_entry_points
from beaker.cache import CacheManager
from beaker.util import parse_cache_config_options
from beaker.middleware import SessionMiddleware
from cocktail.events import Event, event_handler
from cocktail.translations import get_language, set_language
from cocktail.controllers import (
    Dispatcher,
    Location,
    folder_publisher,
    try_decode,
    session
)
from cocktail.controllers.asyncupload import AsyncUploadController
from cocktail.controllers.uriutils import percent_encode
from cocktail.persistence import datastore
from woost import app
from woost.models import (
    Item,
    Publishable,
    URI,
    Site,
    ReadPermission,
    ReadTranslationPermission,
    AuthorizationError,
    get_current_user
)
from woost.models.icons import IconResolver
from woost.controllers.asyncupload import async_uploader
from woost.controllers import get_cache_manager, set_cache_manager
from woost.controllers.basecmscontroller import BaseCMSController
from woost.controllers.language import LanguageModule
from woost.controllers.authentication import (
    AuthenticationModule,
    AuthenticationFailedError
)
from woost.controllers.imagescontroller import ImagesController
from woost.controllers.imageeffectscontroller import ImageEffectsController


class CMSController(BaseCMSController):

    application_settings = None

    # Application events
    item_saved = Event(doc = """
        An event triggered after an item is inserted or modified.

        @ivar item: The saved item.
        @type item: L{Item<woost.models.Item>}

        @ivar user: The user who saved the item.
        @type user: L{User<woost.models.user.User>}

        @ivar is_new: True for an insertion, False for a modification.
        @type is_new: bool

        @ivar change: The revision describing the changes to the item.
        @type change: L{Change<woost.models.Change>}
        """)

    item_deleted = Event(doc = """
        An event triggered after an item is deleted.

        @ivar item: The deleted item.
        @type item: L{Item<woost.models.Item>}

        @ivar user: The user who deleted the item.
        @type user: L{User<woost.models.user.User>}

        @ivar change: The revision describing the changes to the item.
        @type change: L{Change<woost.models.Change>}
        """)

    producing_output = Event(doc = """
        An event triggered to allow setting site-wide output for controllers.

        @ivar controller: The controller that is producing the output.
        @type controller: L{BaseCMSController
                            <woost.controllers.basecmscontroller
                            .BaseCMSController>}

        @ivar output: The output for the controller. Event handlers can modify
            it as required.
        @type output: dict
        """)

    # Application modules
    LanguageModule = LanguageModule
    AuthenticationModule = AuthenticationModule

    # Webserver configuration
    virtual_path = "/"

    # Enable / disable confirmation dialogs when closing an edit session. This
    # setting exists mainly to disable the dialogs on selenium test runs.
    closing_item_requires_confirmation = True

    # A dummy controller for CherryPy, that triggers the cocktail dispatcher.
    # This is done so dynamic dispatching (using the resolve() method of
    # request handlers) can depend on session setup and other requirements
    # being available beforehand.
    class ApplicationContainer(object):

        def __init__(self, cms):
            self.__cms = cms
            self.__dispatcher = Dispatcher()

            # Set the default location for file-based sessions
            sconf = session.config
            using_file_sessions = (sconf.get("session.type") == "file")
            using_dbm_sessions = (sconf.get("session.type") == "dbm")
            lock_dir_missing = (
                not using_file_sessions
                and not sconf.get("session.lock_dir")
            )
            data_dir_missing = (
                (
                    using_file_sessions
                    or (using_dbm_sessions and not
                        sconf.get("session.dbm_dir"))
                )
                and not sconf.get("session.data_dir")
            )

            if lock_dir_missing or data_dir_missing:

                session_path = app.path("sessions")
                if not os.path.exists(session_path):
                    os.mkdir(session_path)

                if lock_dir_missing:
                    sconf["session.lock_dir"] = session_path

                if data_dir_missing:
                    sconf["session.data_dir"] = session_path

            if not sconf.get("session.secret"):

                session_key_path = app.path(".session_key")
                if os.path.exists(session_key_path):
                    with open(session_key_path, "r") as session_key_file:
                        session_key = session_key_file.readline()
                else:
                    with open(session_key_path, "w") as session_key_file:
                        session_key = sha("".join(
                            choice(ascii_letters)
                            for i in range(10)
                        )).hexdigest()
                        session_key_file.write(session_key)

                sconf["session.secret"] = session_key

            # If the cache manager doesn't exist, create it
            if not get_cache_manager():
                cache_path = app.path('cache')

                if not os.path.exists(cache_path):
                    os.mkdir(cache_path)

                cache_manager = CacheManager(
                    **parse_cache_config_options({
                        'cache.lock_dir': cache_path,
                        'cache.regions': 'woost_cache',
                        'cache.woost_cache.type': 'memory'
                    })
                )
                set_cache_manager(cache_manager)

            # Create the folders for uploaded files
            upload_path = app.path("upload")
            if not os.path.exists(upload_path):
                os.mkdir(upload_path)

            temp_path = app.path("upload", "temp")
            if not os.path.exists(temp_path):
                os.mkdir(temp_path)

            async_uploader.temp_folder = temp_path

        @cherrypy.expose
        def default(self, *args, **kwargs):
            # All requests are forwarded to the nested dispatcher:
            return self.__dispatcher.respond(args, self.__cms)

        # Static resources
        resources = folder_publisher(
            resource_filename("woost.views", "resources")
        )

        cocktail = folder_publisher(
            resource_filename("cocktail.html", "resources")
        )

    def session_middleware(self, app):
        return SessionMiddleware(app, session.config)

    def __init__(self, *args, **kwargs):

        BaseCMSController.__init__(self, *args, **kwargs)

        self.language = self.LanguageModule(self)
        self.authentication = self.AuthenticationModule(self)

    def run(self, block = True):

        self.mount()

        if hasattr(cherrypy.engine, "signal_handler"):
            cherrypy.engine.signal_handler.subscribe()

        if hasattr(cherrypy.engine, "console_control_handler"):
            cherrypy.engine.console_control_handler.subscribe()

        cherrypy.engine.start()

        if block:
            cherrypy.engine.block()
        else:
            cherrypy.engine.wait(cherrypy.engine.states.STARTED)

    def mount(self):

        app = cherrypy.Application(self.ApplicationContainer(self))

        # Session middleware
        app.wsgiapp.pipeline.append(
            ("beaker_session", self.session_middleware)
        )

        return cherrypy.tree.mount(
            app,
            self.virtual_path,
            self.application_settings
        )

    def resolve(self, path):

        # Allow application modules (ie. language) to process the URI before
        # resolving the requested publishable item
        self._process_path(path)

        site = Site.main
        request = cherrypy.request
        unicode_path = [try_decode(step) for step in path]

        # Item resolution
        path_resolution = site.resolve_path(unicode_path)

        if path_resolution:
            publishable = path_resolution.item

            for step in path_resolution.matching_path:
                path.pop(0)

            self.canonical_redirection(path_resolution)
        else:
            publishable = site.home

        self.context["publishable"] = publishable

        # HTTP/HTTPS check
        self._apply_https_policy(publishable)

        # Controller resolution
        controller = publishable.resolve_controller()

        if controller is None:
            raise cherrypy.NotFound()

        # Add the selected language to the current URI
        if publishable.per_language_publication:
            if not request.language_specified:
                location = Location.get_current()
                location.path_info = "/" + get_language() + location.path_info
                location.go()

        # Remove the language selection from the current URI
        elif request.language_specified:
            location = Location.get_current()
            location.path_info = \
                "/" + "/".join(location.path_info.strip("/").split("/")[1:])
            location.go()

        return controller

    def canonical_redirection(self, path_resolution):
        """Redirect the current request to the canonical URL for the selected
        publishable element.
        """
        site = Site.main
        publishable = path_resolution.item

        # Find the canonical path for the element
        canonical_path = site.get_path(publishable)

        if canonical_path is None:
            return

        canonical_path = canonical_path.strip("/")
        canonical_path = (
            canonical_path.split("/")
            if canonical_path
            else []
        )

        # The current request matches the canonical path, do nothing
        if canonical_path == path_resolution.matching_path:
            return

        canonical_uri = "".join(
            percent_encode(c) for c in "/" + "/".join(
                step for step in (
                    canonical_path
                    + path_resolution.extra_path
                )
            )
        )

        if publishable.per_language_publication:
            canonical_uri = \
                self.language.translate_uri(canonical_uri)

        if cherrypy.request.query_string:
            canonical_uri = canonical_uri + \
                "?" + cherrypy.request.query_string

        raise cherrypy.HTTPRedirect(canonical_uri, status = 301)

    def _process_path(self, path):

        # Invoke the language module to set the active language
        self.language.process_request(path)

    def uri(self, publishable, *args, **kwargs):
        """Obtains the canonical absolute URI for the given item.

        @param publishable: The item to obtain the canonical URI for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param args: Additional path components to append to the produced URI.
        @type args: unicode

        @param kwargs: Key/value pairs to append to the produced URI as query
            string parameters.
        @type kwargs: (unicode, unicode)

        @return: The item's canonical URI, or None if no matching URI can be
            constructed.
        @rtype: unicode
        """
        warn(
            "CMS.uri() is deprecated, use Publishable.get_uri() instead",
            DeprecationWarning,
            stacklevel = 2
        )

        # User defined URIs
        if isinstance(publishable, URI):
            uri = publishable.uri

        # Regular elements
        else:
            uri = Site.main.get_path(publishable)

            if uri is not None:

                if publishable.per_language_publication:
                    uri = self.language.translate_uri(uri)

                uri = self.application_uri(uri, *args, **kwargs)

        if uri:
            uri = "".join(percent_encode(c) for c in uri)

        return uri

    def translate_uri(self, path = None, language = None):
        return self.application_uri(
            self.language.translate_uri(path = path, language = language)
        )

    def image_uri(self, element, factory = "default"):

        if not isinstance(element, (int, basestring)):
            if isinstance(element, type) \
            or not getattr(element, "is_inserted", False):
                element = element.full_name
            elif hasattr(element, "id"):
                element = element.id

        return self.application_uri("images", element, factory)

    def validate_publishable(self, publishable):

        if not publishable.is_published():
            raise cherrypy.NotFound()

        user = get_current_user()

        user.require_permission(ReadPermission, target = publishable)
        user.require_permission(
            ReadTranslationPermission,
            language = get_language()
        )

    @event_handler
    def handle_traversed(cls, event):

        datastore.sync()

        cms = event.source

        cms.context.update(
            cms = cms,
            publishable = None
        )

        # Set the default language as soon as possible
        language = cms.language.infer_language()
        set_language(language)

        # Invoke the authentication module
        cms.authentication.process_request()

    @event_handler
    def handle_before_request(cls, event):

        cms = event.source

        # Validate access to the requested item
        publishable = cms.context.get("publishable")
        if publishable is not None:
            cms.validate_publishable(publishable)

            # Set the content type and encoding
            content_type = publishable.mime_type
            if content_type:
                encoding = publishable.encoding
                if encoding:
                    content_type += ";charset=" + encoding
                cherrypy.response.headers["Content-Type"] = content_type

            # TODO: encode / decode the request based on the 'encoding' member?

    @event_handler
    def handle_producing_output(cls, event):
        # Set application wide output parameters
        cms = event.source
        event.output.update(
            cms = cms,
            site = Site.main,
            user = get_current_user(),
            publishable = event.controller.context.get("publishable")
        )

    @event_handler
    def handle_exception_raised(cls, event):

        error = event.exception
        controller = event.source

        content_type = cherrypy.response.headers.get("Content-Type")
        pos = content_type.find(";")

        if pos != -1:
            content_type = content_type[:pos]

        if content_type in ("text/html", "text/xhtml"):

            error_page, status = event.source.get_error_page(error)

            if status:
                cherrypy.request.status = status

            if error_page:
                event.handled = True

                # HTTP/HTTPS check
                controller._apply_https_policy(error_page)

                controller.context.update(
                    original_publishable = controller.context["publishable"],
                    publishable = error_page
                )

                response = cherrypy.response
                response.status = status

                error_controller = error_page.resolve_controller()

                # Instantiate class based controllers
                if isinstance(error_controller, type):
                    error_controller = error_controller()
                    error_controller._rendering_format = "html"

                response.body = error_controller()

    def get_error_page(self, error):
        """Produces a custom error page for the indicated exception.

        @param error: The exception to describe.
        @type error: Exception

        @return: A tuple comprised of a publishable item to delegate to and an
            HTTP status to set on the response. Either component can be None,
            so that no custom error page is shown, or that the status is not
            changed, respectively.
        @rtype: (L{Document<woost.models.Document>}, int)
        """
        site = Site.main
        is_http_error = isinstance(error, cherrypy.HTTPError)

        # Page not found
        if is_http_error and error.status == 404:
            return site.not_found_error_page, 404

        # Access forbidden:
        # The default behavior is to show a login page for anonymous users, and
        # a 403 error message for authenticated users.
        elif is_http_error and error.status == 403 \
        or isinstance(error, (AuthorizationError, AuthenticationFailedError)):
            if get_current_user().anonymous:

                publishable = self.context["publishable"]

                while publishable is not None:
                    login_page = publishable.login_page
                    if login_page is not None:
                        return login_page, 403
                    publishable = publishable.parent

                return site.login_page, 403
            else:
                return site.forbidden_error_page, 403

        # Generic error
        elif is_http_error and error.status == 500 \
        or not is_http_error:
            return site.generic_error_page, 500

        return None, None

    def _apply_https_policy(self, publishable):

        site = Site.main
        policy = site.https_policy

        if policy == "always":
            Location.require_https()
        elif policy == "never":
            Location.require_http()
        elif policy == "per_page":
            if publishable.requires_https \
            or not get_current_user().anonymous:
                Location.require_https()
            elif not site.https_persistence:
                Location.require_http()

    @event_handler
    def handle_after_request(cls, event):
        datastore.abort()

    images = ImagesController
    image_effects = ImageEffectsController

    async_upload = AsyncUploadController()
    async_upload.uploader = async_uploader




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from __future__ import with_statement
from woost.tests.models.basetestcase import BaseTestCase


class DraftTestCase(BaseTestCase):

    def test_indexing_enabled_for_new_drafts(self):

        from cocktail import schema
        from woost.models import Item

        class Foo(Item):
            spam = schema.Integer(indexed = True, unique = True)

        foo = Foo()
        foo.is_draft = True
        foo.spam = 4
        foo.insert()
        assert set(Foo.spam.index.items()) == set([(4, foo.id)])

    def test_indexing_disabled_for_draft_copies(self):

        from cocktail import schema
        from woost.models import Item

        class Foo(Item):
            spam = schema.Integer(indexed = True, unique = True)

        source = Foo()
        source.spam = 4
        source.insert()
        assert set(Foo.spam.index.items()) == set([(4, source.id)])

        foo = Foo()
        foo.draft_source = source
        foo.spam = 3
        foo.insert()

        assert set(Foo.spam.index.items()) == set([(4, source.id)])

        foo.spam = 2
        assert set(Foo.spam.index.items()) == set([(4, source.id)])

    def test_versioning_disabled_for_drafts(self):

        from woost.models import Document, changeset_context

        with changeset_context() as insertion_cs:
            doc = Document()
            doc.is_draft = True
            doc.insert()

        assert not insertion_cs.changes

        with changeset_context() as modification_cs:
            doc.set("title", "Hello, world!", "en")

        assert not modification_cs.changes

        with changeset_context() as deletion_cs:
            doc.delete()

        assert not deletion_cs.changes

    def test_copy_draft(self):

        from woost.models import Document

        # Create a source item
        doc = Document()
        doc.set("title", "Test item", "en")
        doc.insert()

        # Create a draft for the item
        draft = doc.make_draft()
        draft.insert()
        assert draft.is_draft
        assert draft.draft_source is doc
        assert draft in doc.drafts
        assert not doc.id == draft.id
        assert doc.get("title", "en") == draft.get("title", "en")

        # Modify the draft
        draft.set("title", "Modified test item", "en")
        assert not doc.get("title", "en") == draft.get("title", "en")
        draft.confirm_draft()

        # Confirm it
        assert draft.id not in Document.index
        assert not draft in doc.drafts
        assert doc.get("title", "en") == "Modified test item"

    def test_unique_validation_enabled_for_new_drafts(self):

        from cocktail import schema
        from cocktail.persistence.persistentobject import UniqueValueError
        from woost.models import Item

        class Foo(Item):
            spam = schema.Integer(indexed = True, unique = True)

        source = Foo()
        source.spam = 4
        source.insert()

        foo = Foo()
        foo.is_draft = True
        foo.spam = 4

        assert len([error
            for error in Foo.get_errors(foo)
            if isinstance(error, UniqueValueError)
            and error.member is Foo.spam
        ]) == 1

    def test_unique_validation_disabled_for_draft_copies(self):

        from cocktail import schema
        from cocktail.persistence.persistentobject import UniqueValueError
        from woost.models import Item

        class Foo(Item):
            spam = schema.Integer(indexed = True, unique = True)

        source = Foo()
        source.spam = 4
        source.insert()

        foo = source.make_draft()

        assert len([error
            for error in Foo.get_errors(foo)
            if isinstance(error, UniqueValueError)
            and error.member is Foo.spam
        ]) == 0

    def test_no_bidirectionality_in_default_references_for_draft_copies(self):

        from cocktail import schema
        from woost.models import Item

        class Foo(Item):
            pass

        Foo.add_member(
            schema.Reference("parent",
                type = Foo,
                bidirectional = True
            )
        )

        Foo.add_member(
            schema.Collection("children",
                items = schema.Reference(type = Foo),
                bidirectional = True
            )
        )

        x = Foo()
        Foo.default_parent = x
        y = Foo()
        y2 = y.make_draft()
        assert y2 not in x.children




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from urllib import urlopen
from httplib import HTTPConnection
from time import mktime, strptime
from cStringIO import StringIO
import Image
from woost.models.uri import URI
from woost.models.rendering.contentrenderer import ContentRenderer
from woost.models.rendering.contentrenderersregistry import content_renderers


class ImageURIRenderer(ContentRenderer):

    def can_render(self, item):
        return isinstance(item, URI) and item.resource_type == "image"

    def get_item_uri(self, item):
        return item.uri

    def render(self, item):

        # Open the remote resource
        uri = self.get_item_uri(item)
        http_resource = urlopen(uri)

        # Wrap image data in a buffer
        # (the object returned by urlopen() doesn't support seek(), which is
        # required by PIL)
        buffer = StringIO()
        buffer.write(http_resource.read())
        buffer.seek(0)

        return Image.open(buffer)

    def last_change_in_appearence(self, item):

        uri = self.get_item_uri(item)
        urlparts = urlsplit(uri)
        host = urlparts[1]
        path = urlparts[2] + urlparts[3] + urlparts[4]

        http_conn = HTTPConnection(host)
        http_conn.request("HEAD", path)
        http_date = http_conn.getresponse().getheader("last-modified")
        http_conn.close()

        if http_date:
            return mktime(strptime(http_date, "%a, %d %b %Y %H:%M:%S %Z"))


content_renderers.register(ImageURIRenderer())




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.events import event_handler
from cocktail import schema
from cocktail.schema.expressions import NegativeExpression
from cocktail.controllers.parameters import SessionParameterSource
from woost.models import (
    Item,
    ChangeSet,
    Language,
    get_current_user,
    ReadHistoryPermission,
    ChangeSetPermissionExpression
)
from woost.models.userfilter import (
    ChangeSetActionFilter,
    ChangeSetTargetFilter
)
from woost.controllers.backoffice.editstack import SelectionNode
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.controllers.backoffice.usercollection \
        import BackOfficeUserCollection


class ChangeLogController(BaseBackOfficeController):

    view_class = "woost.views.BackOfficeChangeLogView"

    def __call__(self, *args, **kwargs):

        rel = cherrypy.request.params.get("ItemSelector-select")

        # Open the item selector
        if rel:

            # Load persistent collection parameters before redirecting
            self.user_collection

            pos = rel.find("-")
            root_content_type_name = rel[:pos]
            selection_parameter = str(rel[pos + 1:])

            for content_type in Item.schema_tree():
                if content_type.full_name == root_content_type_name:

                    edit_stacks_manager = self.context["edit_stacks_manager"]
                    edit_stack = edit_stacks_manager.current_edit_stack

                    if edit_stack is None:
                        edit_stack = edit_stacks_manager.create_edit_stack()
                        edit_stacks_manager.current_edit_stack = edit_stack

                    node = SelectionNode()
                    node.content_type = content_type
                    node.selection_parameter = selection_parameter
                    edit_stack.push(node)
                    raise cherrypy.HTTPRedirect(node.uri(
                        selection = self.params.read(
                            schema.String(selection_parameter)
                        ),
                        client_side_scripting = self.client_side_scripting
                    ))

        return BaseBackOfficeController.__call__(self, *args, **kwargs)

    @event_handler
    def handle_traversed(cls, event):
        get_current_user().require_permission(ReadHistoryPermission)

    @cached_getter
    def user_collection(self):

        class ChangeLogUserCollection(BackOfficeUserCollection):
            persistence_prefix = "changelog"
            schema = self.content_schema
            default_order = [NegativeExpression(ChangeSet.id)]
            allow_sorting = False
            allow_type_selection = False
            allow_language_selection = False
            available_languages = Language.codes

            @cached_getter
            def available_user_filters(self):
                filters = BackOfficeUserCollection.available_user_filters(self)
                cs_action_filter = ChangeSetActionFilter()
                cs_action_filter.id = "member-action"
                cs_target_filter = ChangeSetTargetFilter()
                cs_target_filter.id = "member-changes"
                filters.append(cs_action_filter)
                filters.append(cs_target_filter)
                return filters

        user_collection = ChangeLogUserCollection(ChangeSet)
        user_collection.params.source = SessionParameterSource(
            key_prefix = user_collection.persistence_prefix
        )
        user_collection.add_base_filter(
            ChangeSetPermissionExpression(get_current_user())
        )
        return user_collection

    @cached_getter
    def content_adapter(self):
        return schema.Adapter()

    @cached_getter
    def content_schema(self):
        content_schema = schema.Schema("Changelog")
        content_schema = self.content_adapter.export_schema(
            ChangeSet,
            content_schema
        )
        content_schema.add_member(
            schema.Member("action"),
            before = "changes"
        )

        changes = content_schema["changes"]
        changes.listed_by_default = True
        changes.searchable = True
        return content_schema

    @cached_getter
    def search_expanded(self):
        return bool(
            self.user_collection.user_filters
            or self.params.read(schema.Boolean("search_expanded"))
        )

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            user_collection = self.user_collection,
            search_expanded = self.search_expanded
        )
        return output

    @cached_getter
    def action(self):
        """The user action selected by the current HTTP request.
        @type: L{UserAction<woost.controllers.backoffice.useractions.UserAction>}
        """
        return self._get_user_action()

    @cached_getter
    def ready(self):
        return self.action is not None

    def submit(self):
        self._invoke_user_action(self.action, [])




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			May 2010
"""

import cherrypy
from woost.controllers import BaseCMSController

class FirstChildRedirectionController(BaseCMSController):

    def __call__(self, *args, **kwargs):

        publishable = self.context["publishable"]

        if hasattr(publishable,"children"):

            for child in publishable.children:
                if child.is_accessible():
                    uri = self.context["cms"].uri(child)
                    raise cherrypy.HTTPRedirect(uri)

        raise cherrypy.HTTPError(404)



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from cocktail.schema import Collection, Reference
from cocktail.html import Element, templates

Form = templates.get_class("cocktail.html.Form")


class ContentForm(Form):

    table_layout = False

    def __init__(self, *args, **kwargs):
        Form.__init__(self, *args, **kwargs)
        self.set_member_type_display(Reference, self._get_reference_display)

    def _resolve_member_display(self, obj, member):

        display = getattr(member, "edit_control", None)

        if display is None:
            display = Form._resolve_member_display(self, obj, member)

        return display

    def _get_reference_display(self, obj, member):
        if member.class_family is not None:
            display = templates.new("woost.views.ContentTypePickerDropdown")
            display.content_type_picker.root = member.class_family
        else:
            display = "woost.views.ItemSelector"

        return display




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
import cherrypy
from cocktail.events import event_handler
from cocktail.translations import translations
from woost.controllers.backoffice.editstack import EditNode
from woost.controllers.notifications import notify_user


class ExtensionEditNode(EditNode):

    def item_saved_notification(self, is_new, change):
        if change and "enabled" in change.changed_members:
            notify_user(
                translations(
                    "woost.controllers.backoffice.ExtensionEditNode "
                    "%s extension needs reloading"
                    % ("enabled" if self.item.enabled else "disabled"),
                    extension = self.item
                ),
                "notice",
                transient = False
            )
        else:
            EditNode.item_saved_notification(self, is_new, change)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import cached_getter
from woost.extensions.usermodels.usermembereditnode import UserMemberEditNode


class UserModelEditNode(UserMemberEditNode):

    @cached_getter
    def form_adapter(self):
        adapter = UserMemberEditNode.form_adapter(self)
        adapter.exclude([
            "explanation",
            "member_indexed",
            "member_translated",
            "member_descriptive",
            "member_member_group",
            "member_searchable",
            "member_search_control",
            "member_editable",
            "member_listed_by_default",
            "member_edit_control",
            "member_required",
            "member_unique"
        ])
        return adapter




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
import re
import htmlentitydefs
from difflib import _mdiff

def html_diff(from_sequence, to_sequence, linejunk = None, charjunk = None):

    diffs = _mdiff(
        from_sequence,
        to_sequence,
        linejunk = linejunk,
        charjunk = charjunk
    )

    from_list = []
    to_list = []

    for from_data, to_data, flag in diffs:
        from_list.append(_markup(_unescape(from_data[1])))
        to_list.append(_markup(_unescape(to_data[1])))

    return from_list, to_list

def _markup(data):
    return data \
        .replace("&", "&amp;") \
        .replace(">", "&gt;") \
        .replace("<", "&lt;") \
        .replace('\t', '&nbsp;' * 4) \
        .replace(' ', '&nbsp;') \
        .replace('\0+', '<span class="diff_add">') \
        .replace('\0-', '<span class="diff_sub">') \
        .replace('\0^', '<span class="diff_changed">') \
        .replace('\1', '</span>')

def _unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)

if __name__ == "__main__":
    from_seq = ["Hello universe!"]
    to_seq = ["Hello world!"]
    diffs = html_diff(from_seq, to_seq)
    print diffs

